package impl.evaluation;

import impl.indexing.CorpusIndexer;
import impl.indexing.parsing.OpusCorpusParser;
import impl.model.CorpusService;
import impl.indexing.LexiconIndexer;
import impl.model.Database;
import impl.model.Environment;

import java.io.File;

public class Demo {
	private static CorpusIndexer corpusIndexer;

	public static void main(String[] args) throws InterruptedException {
		Database database = new Database(Environment.getInstance().getProperty("database"));
		LexiconIndexer lexiconIndexer = new LexiconIndexer(new File(Environment.getInstance().getProperty("lexicaDirectory")), database);
		lexiconIndexer.fsToDb();
		corpusIndexer = new CorpusIndexer(new CorpusService(database));
		corpusIndexer.register("Opus", new File(Environment.getInstance().getProperty("opusParallelCorporaDirectory")), new OpusCorpusParser());
		corpusIndexer.register("Opus", new File(Environment.getInstance().getProperty("opusParallelCorporaDirectory")), new OpusCorpusParser());
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			public void run() {
				System.out.println("Preparing to terminate...");
				corpusIndexer.terminate();
				System.out.println("Terminating");
			}
		}));
		synchronized (database) {
			corpusIndexer.start();
		}
		synchronized (database) {
			SearchService searchService = new SearchService(database);
			WebInterface webInterface = new WebInterface(searchService, corpusIndexer);
		}
	}
}
