package impl.evaluation;

import impl.language.Language;
import impl.language.Translator;
import impl.model.*;

import java.io.*;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import static java.sql.ResultSet.TYPE_FORWARD_ONLY;

public class SearchService {
	private Database database;
	private Translator translator;
	private int numExpansions;
	private int numExpansionDocuments;
	private int numExpansionTerms;

	private int parameterIterator;

	public SearchService(Database database) {
		this.database = database;
		translator = new Translator(database);
		numExpansionDocuments = 0;
		numExpansionTerms = 0;
	}

	public SearchReport perform(Query query, Language targetLanguage, boolean grammar, Translator.Pruning pruning) {
		Language sourceLanguage = query.getLanguage();
		SearchReport report;
		if (!sourceLanguage.equals(targetLanguage)) {
			report = new SearchReport(database, grammar);
			for (Query translation : translator.translate(query, targetLanguage, grammar, pruning)) {
				//System.out.println(translation + " " + targetLanguage);
				report.expand(perform(translation, translation.getLanguage(), grammar, pruning));
			}
		}
		else {
			ResultSet retrieved = retrieve(query, grammar);
			report = new SearchReport(database, grammar, query, retrieved);
			for (int i = 0; i < numExpansions; i++) {
				Query expansion = expand(query, retrieved);
				report = new SearchReport(database, grammar, expansion, retrieve(query, grammar));
			}
		}
		try {
			//report.orderByCosineSimilarity();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		return report;
	}

	private Query expand(Query query, ResultSet retrieved) {
		return null;
	}

	public ResultSet retrieve(Query query, boolean grammar) {
		if (query.getVector().size() == 0) {
			return null;
		}
		try {
			File sqlFile = new File("db/search.sql");
			FileReader fr = new FileReader(sqlFile);
			BufferedReader br = new BufferedReader(fr);
			StringBuffer sb = new StringBuffer();
			String str;
			while ((str = br.readLine()) != null) {
				sb.append(str + "\n");
			}
			br.close();
			fr.close();
			parameterIterator = 0;
			String sql = sb.toString()
					.replaceAll(":language", "?")
					.replaceAll(":terms", makePlaceholders(query.getVector().size(), grammar));
			//System.out.println(sql);
			List<SearchResult> results = new ArrayList<SearchResult>();
			PreparedStatement ps = database.getConnection().prepareStatement(sql, TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
			ps.setFetchSize(10);
			addParameters(ps, query, grammar);
			return ps.executeQuery();
			/*
			VectorSpace vs = new VectorSpace(database);
			CorpusService cs = new CorpusService(database);
			Corpus corpus;
			while (rs.next()) {
				//System.out.println(rs.getInt(1) + " " + rs.getDouble(2));
				corpus = cs.getCorpus(rs.getInt(1));
				//System.out.println(query.getLanguage() + " " + corpus.getLanguage());
				results.add(new SearchResult(corpus, vs.cosSim(query, corpus, grammar)));
			}
			return results;
			*/
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

	private String makePlaceholders(int len, boolean grammar) {
		if (len < 1) {
			//throw new RuntimeException("No tokens");
			return "1=1";
		}
		else {
			StringBuilder sb = new StringBuilder();
			if (grammar) {
				sb.append("((t.lemma=? AND t.pos=?)");
				for (int i = 1; i < len; i++) {
					sb.append(" OR (t.lemma=? AND t.pos=?)");
				}
			}
			else {
				sb.append("t.lemma IN(?");
				for (int i = 1; i < len; i++) {
					sb.append(",?");
				}
			}
			sb.append(")");
			return sb.toString();
		}
	}

	private void addParameters(PreparedStatement ps, Query query, boolean grammar) throws SQLException {
		ps.setString(++parameterIterator, query.getLanguage().toString());
		for (Map.Entry<Term, Double> entry : query.getVector().entrySet()) {
			ps.setString(++parameterIterator, entry.getKey().lemma);
			if (grammar) {
				ps.setString(++parameterIterator, entry.getKey().pos);
			}
		}
	}

	public void setNumExpansions(int numExpansions) {
		this.numExpansions = numExpansions;
	}

	public void setNumExpansionDocuments(int numExpansionDocuments) {
		this.numExpansionDocuments = numExpansionDocuments;
	}

	public void setNumExpansionTerms(int numExpansionTerms) {
		this.numExpansionTerms = numExpansionTerms;
	}
}
