package impl.evaluation.setup;

import impl.language.Language;
import impl.language.Translator;
import impl.model.Database;
import impl.model.Environment;
import impl.model.Useful;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.math.MathContext;
import java.util.*;

public class Experiment {
	private static Param[] experiments;
	private static Map<Language, String> langNames;
	private static List<Setup.Metric> overall;

	public static void main(String[] args) {
		initStatics();
		overall = new ArrayList<Setup.Metric>();
		//extend(evaluate(experiments[0]));
		//extend(evaluate(experiments[1]));
		//extend(evaluate(experiments[2]));
		//extend(evaluate(experiments[3]));
		//extend(evaluate(experiments[4]));
		extend(evaluate(experiments[5]));
		summarise();
	}

	private static void initStatics() {
		experiments = new Param[]{
				new Param(
						1,
						new String[] { "skrunøkkel" },
						new String[] { "spanner" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1975/72735/4776593.xml.gz",
								"OpenSubtitles2016/xml/no/1985/88930/5524005.xml.gz",
								"OpenSubtitles2016/xml/no/1994/110475/3619905.xml.gz",
								"OpenSubtitles2016/xml/no/1994/110475/3668411.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1996/708929/161920.xml.gz",
								"OpenSubtitles2016/xml/en/1996/708929/4049780.xml.gz",
								"OpenSubtitles2016/xml/en/1996/708929/3256070.xml.gz"
						)
				),
				new Param(
						2,
						new String[] { "kattunge" },
						new String[] { "Kätzchen" },
						Language.NO,
						Language.DE,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/2015/4807786/6337966.xml.gz",
								"OpenSubtitles2016/xml/no/1988/95776/5570082.xml.gz",
								"OpenSubtitles2016/xml/no/1956/49833/62623.xml.gz",
								"OpenSubtitles2016/xml/no/1956/49833/5985920.xml.gz",
								"OpenSubtitles2016/xml/no/1956/49452/44799.xml.gz",
								"OpenSubtitles2016/xml/no/1996/117951/3977933.xml.gz",
								"OpenSubtitles2016/xml/no/1996/117247/235713.xml.gz",
								"OpenSubtitles2016/xml/no/1992/105128/168898.xml.gz",
								"OpenSubtitles2016/xml/no/1983/86491/3249682.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/de/2015/4578464/6182537.xml.gz",
								"OpenSubtitles2016/xml/de/2015/4807786/6366136.xml.gz",
								"OpenSubtitles2016/xml/de/2015/3895288/6110224.xml.gz",
								"OpenSubtitles2016/xml/de/2015/3884286/6162818.xml.gz",
								"OpenSubtitles2016/xml/de/2015/3884286/6091532.xml.gz",
								"OpenSubtitles2016/xml/de/2015/3884286/6100836.xml.gz",
								"OpenSubtitles2016/xml/de/1994/111127/4981179.xml.gz",
								"OpenSubtitles2016/xml/de/1986/90756/4432294.xml.gz",
								"OpenSubtitles2016/xml/de/1986/90756/4190772.xml.gz",
								"OpenSubtitles2016/xml/de/1986/90756/204501.xml.gz",
								"OpenSubtitles2016/xml/de/1988/95776/4427934.xml.gz",
								"OpenSubtitles2016/xml/de/1965/59903/5542127.xml.gz",
								"OpenSubtitles2016/xml/de/1965/59903/5549336.xml.gz",
								"OpenSubtitles2016/xml/de/1956/49833/5696346.xml.gz",
								"OpenSubtitles2016/xml/de/1956/49833/62620.xml.gz",
								"OpenSubtitles2016/xml/de/1956/49833/37602.xml.gz",
								"OpenSubtitles2016/xml/de/1956/49833/5696349.xml.gz",
								"OpenSubtitles2016/xml/de/1996/117008/179636.xml.gz",
								"OpenSubtitles2016/xml/de/1996/117951/3698426.xml.gz",
								"OpenSubtitles2016/xml/de/1996/117951/3698430.xml.gz",
								"OpenSubtitles2016/xml/de/1996/117247/5208541.xml.gz",
								"OpenSubtitles2016/xml/de/1996/117247/5208540.xml.gz",
								"OpenSubtitles2016/xml/de/1992/103776/5062376.xml.gz",
								"OpenSubtitles2016/xml/de/1992/103776/51906.xml.gz",
								"OpenSubtitles2016/xml/de/1992/105323/18359.xml.gz",
								"OpenSubtitles2016/xml/de/1992/105323/5727132.xml.gz",
								"OpenSubtitles2016/xml/de/1992/103639/6227506.xml.gz",
								"OpenSubtitles2016/xml/de/1992/105128/167762.xml.gz",
								"OpenSubtitles2016/xml/de/1973/70379/5854464.xml.gz",
								"OpenSubtitles2016/xml/de/1973/70379/178446.xml.gz"
						)
				),
				new Param(
						3,
						new String[] { "stormannsgal" },
						new String[] { "megalómano" },
						Language.NO,
						Language.ES,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1968/62859/4464268.xml.gz",
								"OpenSubtitles2016/xml/no/1996/117247/235713.xml.gz",
								"OpenSubtitles2016/xml/no/1973/70379/45883.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/es/1994/111280/6329064.xml.gz",
								"OpenSubtitles2016/xml/es/1994/110632/199477.xml.gz",
								"OpenSubtitles2016/xml/es/1968/62859/3259275.xml.gz"
						)
				),
				new Param(
						4,
						new String[] { "lånehai" },
						new String[] { "loan shark" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1973/70666/6332608.xml.gz",
								"OpenSubtitles2016/xml/no/1988/95953/132587.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1973/70666/144564.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/150198.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/178996.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/192539.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/193903.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/198240.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3259057.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3266577.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3337492.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3484136.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3484484.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3680365.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3688428.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3688431.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/3772852.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/4019605.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/4103300.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/4194236.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/4582485.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/6267956.xml.gz",
								"OpenSubtitles2016/xml/en/1973/70666/6273567.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3083271.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3142804.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3261803.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3529985.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3650593.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3701261.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/3865340.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/4125358.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/4183335.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/4670289.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/4791057.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/4930410.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5043631.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5067858.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5736508.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5809528.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5809529.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5842904.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/5908640.xml.gz",
								"OpenSubtitles2016/xml/en/1988/95953/6167003.xml.gz"
						)
				),
				new Param(
						5,
						new String[] { "Poirot", "samfunnsparasitt" },
						new String[] { "Poirot", "society", "parasite" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1978/77413/212709.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1978/77413/4493129.xml.gz",
								"OpenSubtitles2016/xml/en/1978/77413/4665195.xml.gz",
								"OpenSubtitles2016/xml/en/1978/77413/4719913.xml.gz",
								"OpenSubtitles2016/xml/en/1978/77413/5121168.xml.gz",
								"OpenSubtitles2016/xml/en/1978/77413/5218334.xml.gz",
								"OpenSubtitles2016/xml/en/1978/77413/5218335.xml.gz"
						)
				),
				new Param(
						6,
						new String[] { "Themsen", "klokkefabrikk" },
						new String[] { "Thames", "clock", "factory" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1964/58331/5616494.xml.gz",
								"OpenSubtitles2016/xml/no/1964/58331/5616496.xml.gz"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1964/58331/103150.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/103648.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/115185.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/242069.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/3101761.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/3965790.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/4646445.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/4699285.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/48863.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/50831.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5252206.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5278174.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5278176.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5460461.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/55711.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5576187.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5576189.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5673710.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/5982929.xml.gz",
								"OpenSubtitles2016/xml/en/1964/58331/76089.xml.gz"
						)
				)/*,
				new Param(
						1,
						new String[] { "poirot", "murderer" },
						new String[] { "poirot", "morder" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1978/77413"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1978/77413"
						),
						1,
						6
				),
				new Param(
						1,
						new String[] { "dødsleie" },
						new String[] { "deathbed" },
						Language.NO,
						Language.EN,
						Arrays.asList(
								"OpenSubtitles2016/xml/no/1985/91867"
						),
						Arrays.asList(
								"OpenSubtitles2016/xml/en/1985/91867",
								"OpenSubtitles2016/xml/en/1988/95159",
								"OpenSubtitles2016/xml/en/1994/583599",
								"OpenSubtitles2016/xml/en/1996/116213",
								"OpenSubtitles2016/xml/en/1985/116477"
						),
						1,
						34
				),
				new Param(
						2,
						"",
						new String[] { "bursdagsfeiring" },
						new String[] { "Geburtstagsfeier" },
						Language.NO,
						Language.DE,
						0,
						0
				),
				new Param(
						3,
						"",
						new String[] { "julestjerne" },
						new String[] { "christmas star" },
						Language.NO,
						Language.EN,
						0,
						0
				)*/
		};
		langNames = new HashMap<Language, String>();
		langNames.put(Language.NO, "Norwegian");
		langNames.put(Language.EN, "English");
		langNames.put(Language.DE, "German");
		langNames.put(Language.ES, "Spanish");
	}

	private static List<Setup.Metric> evaluate(Param param) {
		List<Setup.Metric> results = new ArrayList<Setup.Metric>();
		Setup setup = new Setup(new Database(Environment.getInstance().getProperty("database")), null, false, param);
		results.add(setup.evaluate("reference", true, false, null));
		results.add(setup.evaluate("rev", false, false, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("bal", false, false, Translator.Pruning.BALANCE));
		results.add(setup.evaluate("pos-rev", false, true, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("pos-bal", false, true, Translator.Pruning.BALANCE));

		setup = new Setup(new Database(Environment.getInstance().getProperty("test-db-cws")), new Database(Environment.getInstance().getProperty("test-db-ext")), false, param);
		results.add(setup.evaluate("cws-s-rev", false, false, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("cws-s-bal", false, false, Translator.Pruning.BALANCE));
		results.add(setup.evaluate("cws-s-pos-rev", false, true, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("cws-s-pos-bal", false, true, Translator.Pruning.BALANCE));

		setup = new Setup(new Database(Environment.getInstance().getProperty("test-db-cws-r")), null, true, param);
		results.add(setup.evaluate("cws-n-rev", false, false, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("cws-n-bal", false, false, Translator.Pruning.BALANCE));
		results.add(setup.evaluate("cws-n-pos-rev", false, true, Translator.Pruning.REVERSE));
		results.add(setup.evaluate("cws-n-pos-bal", false, true, Translator.Pruning.BALANCE));

		write(param, results);
		return results;
	}

	private static void extend(List<Setup.Metric> results) {
		boolean isNew;
		for (Setup.Metric res : results) {
			isNew = false;
			for (Setup.Metric sum : overall) {
				if (sum.conf.equals(res.conf)) {
					sum.d1 += res.d1;
					sum.d2 += res.d2;
					sum.tp1 += res.tp1;
					sum.tp2 += res.tp2;
					sum.fp1 += res.fp1;
					sum.fp2 += res.fp2;
					isNew = true;
					break;
				}
			}
			if (isNew) {
				overall.add(res);
			}
		}
	}

	private static void write(Param param, List<Setup.Metric> results) {
		try {
			File out = new File(Environment.getInstance().getProperty("export-tex") + "q" + param.id + ".tex");
			if (!out.exists()) {
				out.createNewFile();
			}
			PrintWriter pw = new PrintWriter(out);
			pw.append("\\begin{table}\n");
			pw.append("\t\\centering\n");
			pw.append("\t\\begin{subtable}{1.0\\textwidth}\n");
			pw.append("\t\t\\centering\n");
			pw.append("\t\t\\begin{tabular}{ | l | r | r | r | r | }\n");
			pw.append("\t\t\t\\hline\n");
			pw.append("\t\t\tConfiguration & Retrieved & Recall & Precision & F\\textsubscript{1}-score \\\\\n");
			pw.append("\t\t\t\\hline\n");
			double p, r, f1, cs;
			for (Setup.Metric metric : results) {
				p = metric.tp1 / (metric.tp1 + metric.fp1);
				r = metric.tp1 / metric.d1;
				f1 = (2.0 * p * r) / (p + r);
				//cs = metric.cs1 / (metric.tp1 + metric.fp1);
				pw.append("\t\t\t" + metric.conf);
				pw.append(" & " + roundSignificantly(metric.tp1 + metric.fp1));
				pw.append(" & " + roundSignificantly(r));
				pw.append(" & " + roundSignificantly(p));
				pw.append(" & " + roundSignificantly(f1));
				//pw.append(" & " + roundSignificantly(cs));
				pw.append(" \\\\\n");
			}
			pw.append("\t\t\t\\hline\n");
			pw.append("\t\t\\end{tabular}\n");
			pw.append("\t\t\\caption{Translation from " + langNames.get(param.lang2) + " to " + langNames.get(param.lang1) + ".}\n");
			pw.append("\t\\label{tab:q" + param.id + "a}\n");
			pw.append("\t\\end{subtable}\n");
			pw.append("\t\\newline\n");
			pw.append("\t\\vspace*{1cm}\n");
			pw.append("\t\\newline\n");
			pw.append("\t\\begin{subtable}{1.0\\textwidth}\n");
			pw.append("\t\t\\centering\n");
			pw.append("\t\t\\begin{tabular}{ | l | r | r | r | r | }\n");
			pw.append("\t\t\t\\hline\n");
			pw.append("\t\t\tConfiguration & Retrieved & Recall & Precision & F\\textsubscript{1}-score \\\\\n");
			pw.append("\t\t\t\\hline\n");
			for (Setup.Metric metric : results) {
				p = metric.tp2 / (metric.tp2 + metric.fp2);
				r = metric.tp2 / metric.d2;
				f1 = (2.0 * p * r) / (p + r);
				//cs = metric.cs2 / (metric.tp2 + metric.fp2);
				pw.append("\t\t\t" + metric.conf);
				pw.append(" & " + roundSignificantly(metric.tp2 + metric.fp2));
				pw.append(" & " + roundSignificantly(r));
				pw.append(" & " + roundSignificantly(p));
				pw.append(" & " + roundSignificantly(f1));
				//pw.append(" & " + roundSignificantly(cs));
				pw.append(" \\\\\n");
			}
			pw.append("\t\t\t\\hline\n");
			pw.append("\t\t\\end{tabular}\n");
			pw.append("\t\t\\caption{Translation from " + langNames.get(param.lang1) + " to " + langNames.get(param.lang2) + ".}\n");
			pw.append("\t\\label{tab:q" + param.id + "b}\n");
			pw.append("\t\\end{subtable}\n");
			pw.append("\t\\newline\n");
			pw.append("\t\\vspace*{1cm}\n");
			pw.append("\t\\newline\n");
			if (param.id == 0) {
				pw.append("\t\\caption{Overall retrieval performance results.}\n");
			}
			else {
				pw.append("\t\\caption{Retrieval performance results when applying Q\\textsubscript{" + param.id + "}.}\n");
			}
			pw.append("\t\\label{tab:q" + param.id + "}\n");
			pw.append("\\end{table}\n");
			pw.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void summarise() {
		try {
			File out = new File(Environment.getInstance().getProperty("export-tex") + "q.tex");
			if (!out.exists()) {
				out.createNewFile();
			}
			PrintWriter pw = new PrintWriter(out);
			pw.append("\\begin{table}\n");
			pw.append("\t\\centering\n");
			pw.append("\t\\begin{tabular}{ | l | l | l | l | l | }\n");
			pw.append("\t\t\\hline\n");
			pw.append("\t\tName & Query a & Language a & Query b & Language b \\\\\n");
			pw.append("\t\t\\hline\n");
			for (Param param : experiments) {
				pw.append("\t\tQ\\textsubscript{" + param.id + "} & \\textit{" + Useful.join(param.query1, " ") + "} & " + langNames.get(param.lang1));
				pw.append(" & \\textit{" + Useful.join(param.query2, " ") + "} & " + langNames.get(param.lang2) + "\\\\\n");
			}
			pw.append("\t\t\\hline\n");
			pw.append("\t\\end{tabular}\n");
			pw.append("\\caption[Queries that were used in document retrieval]{Different pairs of queries that were subject to translation and applied to the document retrieval.}\n");
			pw.append("\t\\label{tab:q}\n");
			pw.append("\\end{table}\n");
			pw.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static class Param {
		public int id;
		public String[] query1;
		public String[] query2;
		public Language lang1;
		public Language lang2;
		public List<String> documents1;
		public List<String> documents2;

		public Param(int id, String[] query1, String[] query2, Language lang1, Language lang2, List<String> documents1, List<String> documents2) {
			this.id = id;
			this.query1 = query1;
			this.query2 = query2;
			this.lang1 = lang1;
			this.lang2 = lang2;
			this.documents1 = documents1;
			this.documents2 = documents2;
		}
	}

	private static String roundSignificantly(double input) {
		try {
			BigDecimal bd = new BigDecimal(input);
			bd = bd.round(new MathContext(4));
			return bd.toString();
		} catch (Exception e) {
			return "0";
		}
	}
}
