package impl.evaluation.setup;

import impl.evaluation.SearchService;
import impl.language.Language;
import impl.language.Translator;
import impl.language.splitting.RanangSplitter;
import impl.language.splitting.Splitter;
import impl.language.splitting.StatisticalSplitter;
import impl.model.*;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Setup {
	//private Database database;
	private Database external;
	private SearchService searchService;
	private VectorSpace vectorSpace;
	private Map<Language, Splitter> splitter;
	private Splitter ranangSplitter;
	private Experiment.Param param;

	public Setup(Database database, Database external, boolean ranang, Experiment.Param param) {
		//this.database = database;
		this.external = external;
		searchService = new SearchService(database);
		vectorSpace = new VectorSpace(database);
		if (ranang) {
			LexiconService ls = new LexiconService(database);
			ranangSplitter = new RanangSplitter(ls.getLexicon(Language.NO));
		}
		else {
			ranangSplitter = null;
		}
		this.param = param;
	}

	public Metric evaluate(String conf, boolean reference, boolean grammar, Translator.Pruning pruning) {
		Splitter splitter1 = ranangSplitter == null || !param.lang1.equals(Language.NO) ? getSplitter(param.lang1) : ranangSplitter;
		Splitter splitter2 = ranangSplitter == null || !param.lang2.equals(Language.NO) ? getSplitter(param.lang2) : ranangSplitter;
		String[] phrase1 = splitter1 == null ? param.query1 : splitter1.process(param.query1);
		String[] phrase2 = splitter2 == null ? param.query2 : splitter2.process(param.query2);
		Query query1;
		Query query2;
		if (reference) {
			query1 = Query.byString(Useful.join(phrase1, " "), param.lang1);
			query2 = Query.byString(Useful.join(phrase2, " "), param.lang2);
		}
		else {
			query1 = Query.byString(Useful.join(phrase2, " "), param.lang2);
			query2 = Query.byString(Useful.join(phrase1, " "), param.lang1);
		}
		SearchReport report1 = searchService.perform(query2, param.lang2, grammar, pruning);
		SearchReport report2 = searchService.perform(query1, param.lang1, grammar, pruning);
		System.out.println("id: " + param.id);
		System.out.println(Arrays.toString(phrase1));
		System.out.println(Arrays.toString(phrase2));
		System.out.println(query1);
		System.out.println(query2);
		System.out.println(conf);
		int tp1 = 0;
		int tp2 = 0;
		int fp1 = 0;
		int fp2 = 0;
		double cs1 = .0;
		double cs2 = .0;
		//Query sample = null;
		List<Query> translations2 = report2.getQueries();
		for (Query translation : translations2) {
			System.out.println(translation.toString());
			//sample = translation;
		}
		//SearchResult sr;
		ResultSet rs;
		//double cs;
		while ((rs = report2.getNextResult()) != null) {
			try {
				System.out.print(rs.getString(2) + "\t");
				if (!param.lang1.toString().equals(rs.getString(3))) {
					continue;
				}
				if (param.documents1.contains(rs.getString(2))) {
					tp1++;
					System.out.print("tp");
				} else {
					fp1++;
					System.out.print("fp");
				}
				//cs = vectorSpace.cosSim(sample, sr.getCorpus(), grammar);
				System.out.println("\t");// + cs);
				//cs1 += cs;
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		List<Query> translations1 = report1.getQueries();
		for (Query translation : translations1) {
			System.out.println(translation.toString());
			//sample = translation;
		}
		while ((rs = report1.getNextResult()) != null) {
			try {
				System.out.print(rs.getString(2) + "\t");
				if (!param.lang2.toString().equals(rs.getString(3))) {
					continue;
				}
				if (param.documents2.contains(rs.getString(2))) {
					tp2++;
					System.out.print("tp");
				} else {
					fp2++;
					System.out.print("fp");
				}
				//cs = vectorSpace.cosSim(sample, sr.getCorpus(), grammar);
				System.out.println("\t");// + cs);
				//cs2 += cs;
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		return new Metric(
				conf,
				param.documents1.size(),
				param.documents2.size(),
				tp1,
				tp2,
				fp1,
				fp2,
				cs1,
				cs2
		);
	}

	private Splitter getSplitter(Language language) {
		if (external == null) {
			return null;
		}
		if (splitter == null) {
			splitter = new HashMap<Language, Splitter>();
		}
		if (!splitter.containsKey(language)) {
			splitter.put(language, new StatisticalSplitter(language, external, true));
		}
		return splitter.get(language);
	}

	public static class Metric {
		public String conf;
		public double d1;
		public double d2;
		public double tp1;
		public double tp2;
		public double fp1;
		public double fp2;
		public double cs1;
		public double cs2;

		public Metric(String conf, int d1, int d2, int tp1, int tp2, int fp1, int fp2, double cs1, double cs2) {
			this.conf = conf;
			this.d1 = d1;
			this.d2 = d2;
			this.tp1 = tp1;
			this.tp2 = tp2;
			this.fp1 = fp1;
			this.fp2 = fp2;
			this.cs1 = cs1;
			this.cs2 = cs2;
		}
	}
}
