package impl.evaluation.splitting;

import impl.language.Language;
import impl.language.splitting.StatisticalSplitter;
import impl.model.Database;
import impl.model.Environment;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class SplitOptimiser {
	public static void main(String[] args) {
		Database database = new Database(Environment.getInstance().getProperty("test-db-ext"));
		//int i = 0;
		//for (Map.Entry<String, String[]> entry : getNorwegianTest().entrySet()) {
		//	if (entry.getValue().length > 1) {
		//		i++;
		//	}
		//}
		//System.out.println(i);
		test(new StatisticalSplitter(Language.NO, database, true), getNorwegianTest());
		//test(new StatisticalSplitter(Language.NO, database, true), getNorwegianFinalTest());
	}

	private static void test(StatisticalSplitter ss, Map<String, String[]> test) {
		double h = .25, k = 9.75, k1, k2, bk = 0;
		double[] s1, s2, bs = new double[2];
		do {
			k1 = k - h;
			k2 = k + h;
			s1 = eval(ss, test, k1);
			s2 = eval(ss, test, k2);
			if (s1[1] > s2[1]) {
				k = k1;
				if (s1[1] > bs[1]) {
					bs = s1;
					bk = k;
				}
			}
			else {
				k = k2;
				if (s2[1] > bs[1]) {
					bs = s2;
					bk = k;
				}
			}
			h *= .75;
			System.out.println(k2 - k1);
		} while(Math.abs(k1 - k2) > .125);
		System.out.println("best_penalty=" + bk + ",f1=" + bs[0] + ",best_accuracy=" + bs[1]);
	}

	private static double[] eval(StatisticalSplitter ss, Map<String, String[]> test, double k) {
		System.out.println("Penalty: " + k);
		ss.setPenalty(k);
		int right = 0, wrong = 0, tp = 0, fp = 0, tn = 0, fn = 0;
		double p, r;
		for (Map.Entry<String, String[]> entry : ss.optimisationProcess(test.keySet(), true).entrySet()) {
		//for (Map.Entry<String, String[]> entry : ss.getReplacements(test.keySet(), true).entrySet()) {
			if (Arrays.deepEquals(test.get(entry.getKey()), entry.getValue())) {
				// tp or tn
				right++;
				if (entry.getKey().equals(test.get(entry.getKey())[0])) {
					tn++;
				}
				else {
					tp++;
				}
			}
			else {
				// fn or fp
				wrong++;
				if (entry.getKey().equals(test.get(entry.getKey())[0])) {
					fn++;
				}
				else {
					fp++;
				}
			}
		}
		//System.out.println(tp + " " + fp + " " + fn + " " + tn);
		p = tp == .0 && fp == .0 ? .0 : tp / (tp + fp);
		r = tp == .0 && fn == .0 ? .0 : tp / (tp + fn);
		//System.out.println(p + " " + r);
		double[] s = new double[]{
				2.0 * ((p * r) / (p + r)),
				(double) right / (double) (right + wrong)
		};
		System.out.println("penalty=" + k + ",f1=" + s[0] + ",accuracy=" + s[1]);
		return s;
	}

	public static Map<String, String[]> getNorwegianTest() {
		Map<String, String[]> test = new HashMap<String, String[]>();
		test.put("dødsleie",                 new String[]{ "død", "s", "leie" });
		test.put("vindustetting",            new String[]{ "vindu", "s", "tetting" });
		test.put("tvangsgifte",              new String[]{ "tvang", "s", "gifte" });
		test.put("verdiløs",                 new String[]{ "verdi", "løs" });
		test.put("fiendesamtale",            new String[]{ "fiende", "samtale" });
		test.put("konfekt",                  new String[]{ "konfekt" });
		test.put("utprøve",                  new String[]{ "ut", "prøve" });
		test.put("kjærlighetsting",          new String[]{ "kjærlighet", "s", "ting" });
		test.put("renselsessenter",          new String[]{ "renselse", "s", "senter" });
		test.put("pikkformet",               new String[]{ "pikk", "formet" });
		test.put("æreskodeks",               new String[]{ "ære", "s", "kodeks" });
		test.put("hormonbehandling",         new String[]{ "hormon", "behandling" });
		test.put("taushetserklæring",        new String[]{ "taushet", "s", "erklæring" });
		test.put("kampredskap",              new String[]{ "kamp", "redskap" });
		test.put("sjøkartrute",              new String[]{ "sjø", "kart", "rute" });
		test.put("vampyrsyn",                new String[]{ "vampyr", "syn" });
		test.put("innendørsjente",           new String[]{ "innendør", "s", "jente" });
		test.put("frokostblandingsboks",     new String[]{ "frokost", "blanding", "s", "boks" });
		test.put("bowlingkonge",             new String[]{ "bowling", "konge" });
		test.put("hasteoperasjon",           new String[]{ "haste", "operasjon" });
		test.put("tinninglapp",              new String[]{ "tinning", "lapp" });
		test.put("oppptakskontor",           new String[]{ "opptak", "s", "kontor" });
		test.put("mast",                     new String[]{ "mast" });
		test.put("minkhanske",               new String[]{ "mink", "hanske" });
		test.put("skoleregister",            new String[]{ "skole", "register" });
		test.put("kledning",                 new String[]{ "kledning" });
		test.put("jobbtittel",               new String[]{ "jobb", "tittel" });
		test.put("tegnefilmserie",           new String[]{ "tegne", "film", "serie" });
		test.put("kondensator",              new String[]{ "kondensator" });
		test.put("universitetslektor",       new String[]{ "universitet", "s", "lektor" });
		test.put("blomsterhandler",          new String[]{ "blomster", "handler" });
		test.put("folkekrig",                new String[]{ "folk", "e", "krig" });
		test.put("dyrebitt",                 new String[]{ "dyr", "e", "bitt" });
		test.put("utenomjordisk",            new String[]{ "utenom", "jordisk" });
		test.put("møkkaverden",              new String[]{ "møkk", "a", "verden" });
		test.put("ishockeybane",             new String[]{ "is", "hockey", "bane" });
		test.put("klammeri",                 new String[]{ "klammeri" });
		test.put("suksessintervju",          new String[]{ "suksess", "intervju" });
		test.put("innkomst",                 new String[]{ "innkomst" });
		test.put("sleiping",                 new String[]{ "sleiping" });
		test.put("hjernebark",               new String[]{ "hjerne", "bark" });
		test.put("frekvensering",            new String[]{ "frekvensering" });
		test.put("kvalitet",                 new String[]{ "kvalitet" });
		test.put("master",                   new String[]{ "master" });
		test.put("kjempetare",               new String[]{ "kjempe", "tare" });
		test.put("pølsebil",                 new String[]{ "pølse", "ball" });
		test.put("mislykket",                new String[]{ "mislykket" });
		test.put("lavtstående",              new String[]{ "lavt", "stående" });
		test.put("snev",                     new String[]{ "snev" });
		test.put("straffbar",                new String[]{ "straffbar" });
		test.put("ihjelslått",               new String[]{ "ihjel", "slått" });
		test.put("kokainmonster",            new String[]{ "kokain", "monster" });
		test.put("dame",                     new String[]{ "dame" });
		test.put("arkivlov",                 new String[]{ "arkiv", "lov" });
		test.put("enebarn",                  new String[]{ "ene", "barn" });
		test.put("mordkartell",              new String[]{ "mord", "kartell" });
		test.put("medieetikk",               new String[]{ "medie", "etikk" });
		test.put("uforsonlig",               new String[]{ "uforsonlig" });
		test.put("ulvegift",                 new String[]{ "ulv", "e", "gift" });
		test.put("hundemat",                 new String[]{ "hund", "e", "mat" });
		test.put("fysikklærer",              new String[]{ "fysikk", "lærer" });
		test.put("kanditat",                 new String[]{ "kandidat" });
		test.put("kirurg",                   new String[]{ "kirurg" });
		test.put("tabbe",                    new String[]{ "tabbe" });
		test.put("voksensykdom",             new String[]{ "voksen", "sykdom" });
		test.put("homseassistent",           new String[]{ "homse", "assistent" });
		test.put("stanging",                 new String[]{ "stanging" });
		test.put("kremet",                   new String[]{ "kremet" });
		test.put("vitaminsuppe",             new String[]{ "vitamin", "suppe" });
		test.put("opptaksbrev",              new String[]{ "opptak", "s", "brev" });
		test.put("lettelse",                 new String[]{ "lettelse" });
		test.put("flaggermushule",           new String[]{ "flaggermus", "hule" });
		test.put("faseforflytter",           new String[]{ "fase", "forflytter" });
		test.put("skrytepave",               new String[]{ "skryt", "e", "pave" });
		test.put("bordkledning",             new String[]{ "bord", "kledning" });
		test.put("horestøv",                 new String[]{ "hore", "støv" });
		test.put("internettleverandør",      new String[]{ "internett", "leverandør" });
		test.put("fatle",                    new String[]{ "fatle" });
		test.put("narkotikasyndikat",        new String[]{ "narkotika", "syndikat" });
		test.put("fyrverkerismugling",       new String[]{ "fyrverkeri", "smugling" });
		test.put("fiskedag",                 new String[]{ "fisk", "e", "dag" });
		test.put("grilloppskrift",           new String[]{ "grill", "oppskrift" });
		test.put("rompetryne",               new String[]{ "rompe", "tryne" });
		test.put("radius",                   new String[]{ "radius" });
		test.put("sjokoladefarget",          new String[]{ "sjokolade", "farget" });
		test.put("smågran",                  new String[]{ "små", "gran" });
		test.put("padling",                  new String[]{ "padling" });
		test.put("innadgående",              new String[]{ "innad", "gående" });
		test.put("flaggermusøre",            new String[]{ "flaggermus", "øre" });
		test.put("finneråte",                new String[]{ "finner", "åte" });
		test.put("palmehave",                new String[]{ "palme", "have" });
		test.put("fallvinkel",               new String[]{ "fall", "vinkel" });
		test.put("strand",                   new String[]{ "strand" });
		test.put("vesteuropeisk",            new String[]{ "vest", "europeisk" });
		test.put("navigasjonsstudium",       new String[]{ "navigasjon", "s", "studium" });
		test.put("bibliotekarfantasi",       new String[]{ "bibliotekar", "fantasi" });
		test.put("sikkerhetsforanstaltning", new String[]{ "sikkerhet", "s", "forantstaltning" });
		test.put("kjemibedrift",             new String[]{ "kjemi", "bedrift" });
		test.put("brynje",                   new String[]{ "brynje" });
		test.put("pornoavhengighet",         new String[]{ "porno", "avhengighet" });
		test.put("vinduskontor",             new String[]{ "vindu", "s", "kontor" });
		test.put("guttebass",                new String[]{ "guttebass" });
		test.put("instinktiv",               new String[]{ "instinktiv" });
		test.put("unndskyld",                new String[]{ "unnskyld" });
		test.put("papirkutt",                new String[]{ "papir", "kutt" });
		test.put("lunsj",                    new String[]{ "lunsj" });
		test.put("feltartilleriavdeling",    new String[]{ "felt", "artilleri", "avdeling" });
		test.put("undertekst",               new String[]{ "under", "tekst" });
		test.put("homsestripper",            new String[]{ "homse", "stripper" });
		test.put("lønningssjekk",            new String[]{ "lønning", "s", "sjekk" });
		test.put("forsvarsstyrke",           new String[]{ "forsvar", "s", "styrke" });
		test.put("trusselvurdering",         new String[]{ "trussel", "vurdering" });
		test.put("bowlingsko",               new String[]{ "bowling", "sko" });
		test.put("rørende",                  new String[]{ "rørende" });
		test.put("fengselsprest",            new String[]{ "fengsel", "s", "prest" });
		test.put("remiks",                   new String[]{ "remiks" });
		test.put("handlingsrom",             new String[]{ "hansling", "s", "rom" });
		test.put("våpenskap",                new String[]{ "våpen", "skap" });
		test.put("tennisgreia",              new String[]{ "tennis", "greia" });
		test.put("venstrehendt",             new String[]{ "venstre", "hendt" });
		test.put("tatoveringsmorder",        new String[]{ "tatovering", "s", "morder" });
		test.put("spillebransje",            new String[]{ "spill", "e", "bransje" });
		test.put("lastesone",                new String[]{ "last", "e", "sone" });
		test.put("krengningsstabilisator",   new String[]{ "krengning", "s", "stabilisator" });
		test.put("rynkete",                  new String[]{ "rynkete" });
		test.put("barkebrød",                new String[]{ "bark", "e", "brød" });
		test.put("vaffelfjell",              new String[]{ "vaffel", "fjell" });
		test.put("energiminister",           new String[]{ "energi", "minister" });
		test.put("ublanda",                  new String[]{ "ublanda" });
		test.put("røkt",                     new String[]{ "røkt" });
		test.put("etternavn",                new String[]{ "etter", "navn" });
		test.put("tjuefire",                 new String[]{ "tjue", "fire" });
		test.put("hjemmesløydlag",           new String[]{ "hjemme", "sløyd", "lag" });
		test.put("satanistrituale",          new String[]{ "satanist", "rituale" });
		test.put("standsperson",             new String[]{ "stand", "s", "person" });
		test.put("våpenfiksert",             new String[]{ "våpen", "fiksert" });
		test.put("jentelig",                 new String[]{ "jentelig" });
		test.put("vinnertall",               new String[]{ "vinner", "tall" });
		test.put("firmahelikopter",          new String[]{ "firma", "helikopter" });
		test.put("transefølelse",            new String[]{ "transe", "følelse" });
		test.put("gnagsår",                  new String[]{ "gnag", "sår" });
		test.put("homsepest",                new String[]{ "homse", "pest" });
		test.put("julebilde",                new String[]{ "jule", "bilde" });
		test.put("markedsføringsbudsjett",   new String[]{ "marked", "s", "føring", "s", "budsjett" });
		test.put("skolebildedag",            new String[]{ "skole", "bilde", "dag" });
		test.put("raner",                    new String[]{ "raner" });
		test.put("trøkke",                   new String[]{ "trøkke" });
		test.put("kriminalassistent",        new String[]{ "kriminal", "assistent" });
		test.put("filmfontene",              new String[]{ "film", "fontene" });
		test.put("vareinnlevering",          new String[]{ "vare", "inn", "levering" });
		test.put("valpeånd",                 new String[]{ "valp", "e", "ånd" });
		test.put("myrvann",                  new String[]{ "myr", "vann" });
		test.put("nabodelstat",              new String[]{ "nabo", "del", "stat" });
		test.put("uladd",                    new String[]{ "uladd" });
		test.put("kjerneverdi",              new String[]{ "kjerneverdi" });
		test.put("veteranklubb",             new String[]{ "veteran", "klubb" });
		test.put("slasket",                  new String[]{ "slasket" });
		test.put("velstekt",                 new String[]{ "vel", "stekt" });
		test.put("bokmesse",                 new String[]{ "bok", "messe" });
		test.put("eldreomsorg",              new String[]{ "eldre", "omsorg" });
		test.put("ølhage",                   new String[]{ "øl", "hage" });
		test.put("vilttilsyn",               new String[]{ "vilt", "tilsyn" });
		test.put("brysom",                   new String[]{ "brysom" });
		test.put("vidunderiig",              new String[]{ "vidunderlig" });
		test.put("pågang",                   new String[]{ "pågang" });
		test.put("våpenangrep",              new String[]{ "våpen", "angrep" });
		test.put("valglokale",               new String[]{ "valg", "lokale" });
		test.put("kemneransatt",             new String[]{ "kemner", "ansatt" });
		test.put("telefonvekking",           new String[]{ "telefon", "vekking" });
		test.put("møllehjul",                new String[]{ "mølle", "hjul" });
		test.put("kjempepopulær",            new String[]{ "kjempe", "populær" });
		test.put("fiaskotur",                new String[]{ "fiasko", "tur" });
		test.put("rampestrek",               new String[]{ "ramp", "e", "strek" });
		test.put("tomatsmørbrød",            new String[]{ "tomat", "smørbrød" });
		test.put("partikellakselerator",     new String[]{ "partikkel", "akselerator" });
		test.put("bryggeri",                 new String[]{ "bryggeri" });
		test.put("uttrykksfull",             new String[]{ "uttrykk", "s", "full" });
		test.put("passform",                 new String[]{ "pass", "form" });
		test.put("karbontest",               new String[]{ "karbon", "test" });
		test.put("romgreier",                new String[]{ "rom", "greier" });
		test.put("ekspertutsagn",            new String[]{ "ekspert", "utsagn" });
		test.put("sneglekampanje",           new String[]{ "snegle", "kampanje" });
		test.put("vegguttak",                new String[]{ "vegg", "uttak" });
		test.put("overformynderi",           new String[]{ "over", "formynderi" });
		test.put("reklameavdeling",          new String[]{ "reklame", "avdeling" });
		test.put("overnattingsbesøk",        new String[]{ "overnatting", "s", "besøk" });
		test.put("hjelpearbeider",           new String[]{ "hjelpe", "arbeider" });
		test.put("skriftiig",                new String[]{ "skriftlig" });
		test.put("referansenummer",          new String[]{ "referanse", "nummer" });
		test.put("hovedfilter",              new String[]{ "hoved", "filter" });
		test.put("grynting",                 new String[]{ "grynting" });
		test.put("kosmetikkslave",           new String[]{ "kosmetikk", "slave" });
		test.put("sjåførskilt",              new String[]{ "sjåfør", "skilt" });
		test.put("kortkunstner",             new String[]{ "kort", "kunstner" });
		test.put("hundepark",                new String[]{ "hund", "e", "park" });
		test.put("midtsommersang",           new String[]{ "midt", "sommer", "sang" });
		test.put("sørstatsrebell",           new String[]{ "sør", "stat", "s", "rebell" });
		test.put("lørdagsmøte",              new String[]{ "lørdag", "s", "møte" });
		test.put("fittedoktor",              new String[]{ "fitte", "doktor" });
		test.put("hærhovedkvarter",          new String[]{ "hær", "hoved", "kvarter" });
		return test;
	}

	public static Map<String, String[]> getNorwegianFinalTest() {
		Map<String, String[]> test = new HashMap<String, String[]>();
		test.put("dødsleie",                 new String[]{ "død", "leie" });
		test.put("vindustetting",            new String[]{ "vindu", "tetting" });
		test.put("tvangsgifte",              new String[]{ "tvang", "gifte" });
		test.put("verdiløs",                 new String[]{ "verdi", "løs" });
		test.put("fiendesamtale",            new String[]{ "fiende", "samtale" });
		test.put("konfekt",                  new String[]{ "konfekt" });
		test.put("utprøve",                  new String[]{ "ut", "prøve" });
		test.put("kjærlighetsting",          new String[]{ "kjærlighet", "ting" });
		test.put("renselsessenter",          new String[]{ "renselse", "senter" });
		test.put("pikkformet",               new String[]{ "pikk", "formet" });
		test.put("æreskodeks",               new String[]{ "ære", "kodeks" });
		test.put("hormonbehandling",         new String[]{ "hormon", "behandling" });
		test.put("taushetserklæring",        new String[]{ "taushet", "erklæring" });
		test.put("kampredskap",              new String[]{ "kamp", "redskap" });
		test.put("sjøkartrute",              new String[]{ "sjø", "kart", "rute" });
		test.put("vampyrsyn",                new String[]{ "vampyr", "syn" });
		test.put("innendørsjente",           new String[]{ "innendør", "jente" });
		test.put("frokostblandingsboks",     new String[]{ "frokost", "blanding", "boks" });
		test.put("bowlingkonge",             new String[]{ "bowling", "konge" });
		test.put("hasteoperasjon",           new String[]{ "haste", "operasjon" });
		test.put("tinninglapp",              new String[]{ "tinning", "lapp" });
		test.put("oppptakskontor",           new String[]{ "opptak", "kontor" });
		test.put("mast",                     new String[]{ "mast" });
		test.put("minkhanske",               new String[]{ "mink", "hanske" });
		test.put("skoleregister",            new String[]{ "skole", "register" });
		test.put("kledning",                 new String[]{ "kledning" });
		test.put("jobbtittel",               new String[]{ "jobb", "tittel" });
		test.put("tegnefilmserie",           new String[]{ "tegne", "film", "serie" });
		test.put("kondensator",              new String[]{ "kondensator" });
		test.put("universitetslektor",       new String[]{ "universitet", "lektor" });
		test.put("blomsterhandler",          new String[]{ "blomster", "handler" });
		test.put("folkekrig",                new String[]{ "folk", "krig" });
		test.put("dyrebitt",                 new String[]{ "dyr", "bitt" });
		test.put("utenomjordisk",            new String[]{ "utenom", "jordisk" });
		test.put("møkkaverden",              new String[]{ "møkk", "verden" });
		test.put("ishockeybane",             new String[]{ "is", "hockey", "bane" });
		test.put("klammeri",                 new String[]{ "klammeri" });
		test.put("suksessintervju",          new String[]{ "suksess", "intervju" });
		test.put("innkomst",                 new String[]{ "innkomst" });
		test.put("sleiping",                 new String[]{ "sleiping" });
		test.put("hjernebark",               new String[]{ "hjerne", "bark" });
		test.put("frekvensering",            new String[]{ "frekvensering" });
		test.put("kvalitet",                 new String[]{ "kvalitet" });
		test.put("master",                   new String[]{ "master" });
		test.put("kjempetare",               new String[]{ "kjempe", "tare" });
		test.put("pølsebil",                 new String[]{ "pølse", "ball" });
		test.put("mislykket",                new String[]{ "mislykket" });
		test.put("lavtstående",              new String[]{ "lavt", "stående" });
		test.put("snev",                     new String[]{ "snev" });
		test.put("straffbar",                new String[]{ "straffbar" });
		test.put("ihjelslått",               new String[]{ "ihjel", "slått" });
		test.put("kokainmonster",            new String[]{ "kokain", "monster" });
		test.put("dame",                     new String[]{ "dame" });
		test.put("arkivlov",                 new String[]{ "arkiv", "lov" });
		test.put("enebarn",                  new String[]{ "ene", "barn" });
		test.put("mordkartell",              new String[]{ "mord", "kartell" });
		test.put("medieetikk",               new String[]{ "medie", "etikk" });
		test.put("uforsonlig",               new String[]{ "uforsonlig" });
		test.put("ulvegift",                 new String[]{ "ulv", "gift" });
		test.put("hundemat",                 new String[]{ "hund", "mat" });
		test.put("fysikklærer",              new String[]{ "fysikk", "lærer" });
		test.put("kanditat",                 new String[]{ "kandidat" });
		test.put("kirurg",                   new String[]{ "kirurg" });
		test.put("tabbe",                    new String[]{ "tabbe" });
		test.put("voksensykdom",             new String[]{ "voksen", "sykdom" });
		test.put("homseassistent",           new String[]{ "homse", "assistent" });
		test.put("stanging",                 new String[]{ "stanging" });
		test.put("kremet",                   new String[]{ "kremet" });
		test.put("vitaminsuppe",             new String[]{ "vitamin", "suppe" });
		test.put("opptaksbrev",              new String[]{ "opptak", "brev" });
		test.put("lettelse",                 new String[]{ "lettelse" });
		test.put("flaggermushule",           new String[]{ "flaggermus", "hule" });
		test.put("faseforflytter",           new String[]{ "fase", "forflytter" });
		test.put("skrytepave",               new String[]{ "skryt", "pave" });
		test.put("bordkledning",             new String[]{ "bord", "kledning" });
		test.put("horestøv",                 new String[]{ "hore", "støv" });
		test.put("internettleverandør",      new String[]{ "internett", "leverandør" });
		test.put("fatle",                    new String[]{ "fatle" });
		test.put("narkotikasyndikat",        new String[]{ "narkotika", "syndikat" });
		test.put("fyrverkerismugling",       new String[]{ "fyrverkeri", "smugling" });
		test.put("fiskedag",                 new String[]{ "fisk", "dag" });
		test.put("grilloppskrift",           new String[]{ "grill", "oppskrift" });
		test.put("rompetryne",               new String[]{ "rompe", "tryne" });
		test.put("radius",                   new String[]{ "radius" });
		test.put("sjokoladefarget",          new String[]{ "sjokolade", "farget" });
		test.put("smågran",                  new String[]{ "små", "gran" });
		test.put("padling",                  new String[]{ "padling" });
		test.put("innadgående",              new String[]{ "innad", "gående" });
		test.put("flaggermusøre",            new String[]{ "flaggermus", "øre" });
		test.put("finneråte",                new String[]{ "finner", "åte" });
		test.put("palmehave",                new String[]{ "palme", "have" });
		test.put("fallvinkel",               new String[]{ "fall", "vinkel" });
		test.put("strand",                   new String[]{ "strand" });
		test.put("vesteuropeisk",            new String[]{ "vest", "europeisk" });
		test.put("navigasjonsstudium",       new String[]{ "navigasjon", "studium" });
		test.put("bibliotekarfantasi",       new String[]{ "bibliotekar", "fantasi" });
		test.put("sikkerhetsforanstaltning", new String[]{ "sikkerhet", "forantstaltning" });
		test.put("kjemibedrift",             new String[]{ "kjemi", "bedrift" });
		test.put("brynje",                   new String[]{ "brynje" });
		test.put("pornoavhengighet",         new String[]{ "porno", "avhengighet" });
		test.put("vinduskontor",             new String[]{ "vindu", "kontor" });
		test.put("guttebass",                new String[]{ "guttebass" });
		test.put("instinktiv",               new String[]{ "instinktiv" });
		test.put("unndskyld",                new String[]{ "unnskyld" });
		test.put("papirkutt",                new String[]{ "papir", "kutt" });
		test.put("lunsj",                    new String[]{ "lunsj" });
		test.put("feltartilleriavdeling",    new String[]{ "felt", "artilleri", "avdeling" });
		test.put("undertekst",               new String[]{ "under", "tekst" });
		test.put("homsestripper",            new String[]{ "homse", "stripper" });
		test.put("lønningssjekk",            new String[]{ "lønning", "sjekk" });
		test.put("forsvarsstyrke",           new String[]{ "forsvar", "styrke" });
		test.put("trusselvurdering",         new String[]{ "trussel", "vurdering" });
		test.put("bowlingsko",               new String[]{ "bowling", "sko" });
		test.put("rørende",                  new String[]{ "rørende" });
		test.put("fengselsprest",            new String[]{ "fengsel", "prest" });
		test.put("remiks",                   new String[]{ "remiks" });
		test.put("handlingsrom",             new String[]{ "hansling", "rom" });
		test.put("våpenskap",                new String[]{ "våpen", "skap" });
		test.put("tennisgreia",              new String[]{ "tennis", "greia" });
		test.put("venstrehendt",             new String[]{ "venstre", "hendt" });
		test.put("tatoveringsmorder",        new String[]{ "tatovering", "morder" });
		test.put("spillebransje",            new String[]{ "spill", "bransje" });
		test.put("lastesone",                new String[]{ "last", "sone" });
		test.put("krengningsstabilisator",   new String[]{ "krengning", "stabilisator" });
		test.put("rynkete",                  new String[]{ "rynkete" });
		test.put("barkebrød",                new String[]{ "bark", "brød" });
		test.put("vaffelfjell",              new String[]{ "vaffel", "fjell" });
		test.put("energiminister",           new String[]{ "energi", "minister" });
		test.put("ublanda",                  new String[]{ "ublanda" });
		test.put("røkt",                     new String[]{ "røkt" });
		test.put("etternavn",                new String[]{ "etter", "navn" });
		test.put("tjuefire",                 new String[]{ "tjue", "fire" });
		test.put("hjemmesløydlag",           new String[]{ "hjemme", "sløyd", "lag" });
		test.put("satanistrituale",          new String[]{ "satanist", "rituale" });
		test.put("standsperson",             new String[]{ "stand", "person" });
		test.put("våpenfiksert",             new String[]{ "våpen", "fiksert" });
		test.put("jentelig",                 new String[]{ "jentelig" });
		test.put("vinnertall",               new String[]{ "vinner", "tall" });
		test.put("firmahelikopter",          new String[]{ "firma", "helikopter" });
		test.put("transefølelse",            new String[]{ "transe", "følelse" });
		test.put("gnagsår",                  new String[]{ "gnag", "sår" });
		test.put("homsepest",                new String[]{ "homse", "pest" });
		test.put("julebilde",                new String[]{ "jule", "bilde" });
		test.put("markedsføringsbudsjett",   new String[]{ "marked", "føring", "budsjett" });
		test.put("skolebildedag",            new String[]{ "skole", "bilde", "dag" });
		test.put("raner",                    new String[]{ "raner" });
		test.put("trøkke",                   new String[]{ "trøkke" });
		test.put("kriminalassistent",        new String[]{ "kriminal", "assistent" });
		test.put("filmfontene",              new String[]{ "film", "fontene" });
		test.put("vareinnlevering",          new String[]{ "vare", "inn", "levering" });
		test.put("valpeånd",                 new String[]{ "valp", "ånd" });
		test.put("myrvann",                  new String[]{ "myr", "vann" });
		test.put("nabodelstat",              new String[]{ "nabo", "del", "stat" });
		test.put("uladd",                    new String[]{ "uladd" });
		test.put("kjerneverdi",              new String[]{ "kjerneverdi" });
		test.put("veteranklubb",             new String[]{ "veteran", "klubb" });
		test.put("slasket",                  new String[]{ "slasket" });
		test.put("velstekt",                 new String[]{ "vel", "stekt" });
		test.put("bokmesse",                 new String[]{ "bok", "messe" });
		test.put("eldreomsorg",              new String[]{ "eldre", "omsorg" });
		test.put("ølhage",                   new String[]{ "øl", "hage" });
		test.put("vilttilsyn",               new String[]{ "vilt", "tilsyn" });
		test.put("brysom",                   new String[]{ "brysom" });
		test.put("vidunderiig",              new String[]{ "vidunderlig" });
		test.put("pågang",                   new String[]{ "pågang" });
		test.put("våpenangrep",              new String[]{ "våpen", "angrep" });
		test.put("valglokale",               new String[]{ "valg", "lokale" });
		test.put("kemneransatt",             new String[]{ "kemner", "ansatt" });
		test.put("telefonvekking",           new String[]{ "telefon", "vekking" });
		test.put("møllehjul",                new String[]{ "mølle", "hjul" });
		test.put("kjempepopulær",            new String[]{ "kjempe", "populær" });
		test.put("fiaskotur",                new String[]{ "fiasko", "tur" });
		test.put("rampestrek",               new String[]{ "ramp", "strek" });
		test.put("tomatsmørbrød",            new String[]{ "tomat", "smørbrød" });
		test.put("partikellakselerator",     new String[]{ "partikkel", "akselerator" });
		test.put("bryggeri",                 new String[]{ "bryggeri" });
		test.put("uttrykksfull",             new String[]{ "uttrykk", "full" });
		test.put("passform",                 new String[]{ "pass", "form" });
		test.put("karbontest",               new String[]{ "karbon", "test" });
		test.put("romgreier",                new String[]{ "rom", "greier" });
		test.put("ekspertutsagn",            new String[]{ "ekspert", "utsagn" });
		test.put("sneglekampanje",           new String[]{ "snegle", "kampanje" });
		test.put("vegguttak",                new String[]{ "vegg", "uttak" });
		test.put("overformynderi",           new String[]{ "over", "formynderi" });
		test.put("reklameavdeling",          new String[]{ "reklame", "avdeling" });
		test.put("overnattingsbesøk",        new String[]{ "overnatting", "besøk" });
		test.put("hjelpearbeider",           new String[]{ "hjelpe", "arbeider" });
		test.put("skriftiig",                new String[]{ "skriftlig" });
		test.put("referansenummer",          new String[]{ "referanse", "nummer" });
		test.put("hovedfilter",              new String[]{ "hoved", "filter" });
		test.put("grynting",                 new String[]{ "grynting" });
		test.put("kosmetikkslave",           new String[]{ "kosmetikk", "slave" });
		test.put("sjåførskilt",              new String[]{ "sjåfør", "skilt" });
		test.put("kortkunstner",             new String[]{ "kort", "kunstner" });
		test.put("hundepark",                new String[]{ "hund", "park" });
		test.put("midtsommersang",           new String[]{ "midt", "sommer", "sang" });
		test.put("sørstatsrebell",           new String[]{ "sør", "stat", "rebell" });
		test.put("lørdagsmøte",              new String[]{ "lørdag", "møte" });
		test.put("fittedoktor",              new String[]{ "fitte", "doktor" });
		test.put("hærhovedkvarter",          new String[]{ "hær", "hoved", "kvarter" });
		return test;
	}
}
