package impl.evaluation.splitting;

import impl.indexing.LexiconIndexer;
import impl.language.Language;
import impl.language.splitting.StatisticalSplitter;
import impl.model.Database;
import impl.model.Environment;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

public class SplitTrainer {
	private static StatisticalSplitter splitter = null;
	private static Map<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> findings;

	public static void main(String[] args) throws InterruptedException {
		Language language = Language.NO;
		Database database = new Database(Environment.getInstance().getProperty("test-db-ext"));
		LexiconIndexer lexiconIndexer = new LexiconIndexer(new File(Environment.getInstance().getProperty("lexicaDirectory")), database);
		lexiconIndexer.fsToDb();
		splitter = new StatisticalSplitter(language, database, false);
		findings = new HashMap<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>();
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
			public void run() {
				System.out.println("Terminating...");
				//splitter.flush();
			}
		}));
		//System.out.println(splitter.getMorphologicalOperation("dødshjem", new String[]{"død", "hjem"}));
		//System.out.println(splitter.getMorphologicalOperation("møkkaverden", new String[]{"møkk", "verden"}));
		//System.out.println(splitter.getMorphologicalOperation("ento", new String[]{"en", "tre", "to"}));
		//System.out.println(splitter.getMorphologicalOperation("spionfilm", new String[]{"spion", "film"}));
		//System.out.println(splitter.getMorphologicalOperation("pauseprat", new String[]{"pause"}));
		//System.out.println(splitter.getMorphologicalOperation("pauseprat", new String[]{"prat"}));
		//for (StatisticalSplitter.MorphologicalOperation mo : splitter.getMorphologicalOperationHeuristics("markedsføringslastesone", new String[]{"marked", "s", "føring", "s", "last", "e", "sone"})) {
		//	System.out.println(mo);
		//}
		iterate(database, language);
	}

	private static void iterate(Database database, Language language) {
		String sql = "SELECT lemma FROM term WHERE language=? GROUP BY lemma ORDER BY RAND() LIMIT 100000;";
		try {
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setString(1, language.toString());
			ResultSet rs = ps.executeQuery();
			int i = 0;
			Set<String> words = new HashSet<String>();
			loop:
			for (;;) {
				for (;;) {
					if (!rs.next()) {
						break loop;
					}
					words.add(rs.getString(1));
					if (++i % 2000 == 0) {
						train(words, i);
						System.out.println(i);
						words = new HashSet<String>();
						continue loop;
					}
				}
			}
			if (words.size() == 0) {
				train(words, i);
				System.out.println(i);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private static void train(Set<String> words, int count) {
		addMops(splitter.trainProcess(words, true));
		SortedSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>> sorted = getSorted();
		print(sorted);
		write(sorted, count);
	}

	private static void addMops(Map<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> mops) {
		for (Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> entry : mops.entrySet()) {
			findings.put(entry.getKey(), findings.containsKey(entry.getKey()) ? findings.get(entry.getKey()).add(entry.getValue()) : entry.getValue());
		}
	}

	private static SortedSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>> getSorted() {
		SortedSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>> sorted = new TreeSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>>(
				new Comparator<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>>() {
					public int compare(Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> o1, Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> o2) {
						int diff = (int) (o2.getValue().improvement - o1.getValue().improvement);
						if (diff != 0) {
							return diff;
						}
						return o2.getKey().hashCode() - o1.getKey().hashCode();
					}
				}
		);
		sorted.addAll(findings.entrySet());
		return sorted;
	}

	private static void print(SortedSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>> sorted) {
		System.out.println(findings);
		System.out.println(sorted);
	}

	private static void write(SortedSet<Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore>> sorted, int count) {
		try {
			File out = new File(Environment.getInstance().getProperty("test-morph.NO"));
			if (!out.exists()) {
				out.createNewFile();
			}
			PrintWriter pw = new PrintWriter(out);
			pw.append(count + "\n");
			for (Map.Entry<StatisticalSplitter.MorphologicalOperation, StatisticalSplitter.GlobalScore> entry : sorted) {
				pw.append(entry.getKey().decompounded + "/" + entry.getKey().compounded + " ");
				pw.append(entry.getValue() + "\n");
			}
			pw.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
