package impl.indexing;

import impl.indexing.parsing.CorpusParser;
import impl.language.Language;
import impl.model.CorpusService;

import java.util.HashMap;
import java.util.Map;

public class CorpusIndexer {
	private CorpusService corpusService;
	private Map<String, IndexerThread> threads;

	public CorpusIndexer(CorpusService corpusService) {
		this.corpusService = corpusService;
		threads = new HashMap<String, IndexerThread>();
	}

	public void register(String name, Object source, CorpusParser cp) throws IllegalArgumentException {
		cp.setSource(source);
		cp.setCorpusService(corpusService);
		threads.put(name, new IndexerThread(cp));
	}

	public void start() {
		for (Map.Entry<String, IndexerThread> it : threads.entrySet()) {
			it.getValue().setPriority(Thread.MIN_PRIORITY);
			it.getValue().start();
		}
	}

	public void terminate() {
		for (Map.Entry<String, IndexerThread> it : threads.entrySet()) {
			it.getValue().terminate();
		}
		for (Map.Entry<String, IndexerThread> it : threads.entrySet()) {
			try {
				it.getValue().join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public CorpusParser.RawCorpusAccess getRawCorpusAccess(String source, String collection, Language language, String title) throws Exception {
		if (!threads.containsKey(source)) {
			return null;
		}
		return threads.get(source).getRawCorpusAccess(collection, language, title);
	}

	private class IndexerThread extends Thread {
		private CorpusParser cp;

		public IndexerThread(CorpusParser cp) {
			this.cp = cp;
		}

		@Override
		public void run() {
			try {
				cp.indexFromSource();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		public void terminate() {
			cp.terminate();
		}

		public CorpusParser.RawCorpusAccess getRawCorpusAccess(String collection, Language language, String title) throws Exception {
			return cp.getRawCorpusAccess(collection, language, title);
		}
	}
}
