package impl.indexing;

import impl.language.Language;
import impl.language.Stopword;
import impl.model.*;

import java.io.*;
import java.sql.SQLException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LexiconIndexer {
	private File rootFolder;
	private LexiconService lexiconService;
	private Map<Language, Stopword> stoppers;

	public LexiconIndexer(File rootFolder, Database database) {
		this.rootFolder = rootFolder;
		lexiconService = new LexiconService(database);
		stoppers = new HashMap<Language, Stopword>();
	}

	public LexiconIndexer(File rootFolder, LexiconService lexiconService) {
		this.rootFolder = rootFolder;
		this.lexiconService = lexiconService;
		stoppers = new HashMap<Language, Stopword>();
	}

	public static void main(String[] args) {
		Database database = new Database(Environment.getInstance().getProperty("database"));
		if (!database.isNew()) {
			System.out.println("Database already exists, not importing any lexica.");
			return;
		}
		//LexiconIndexer lexiconIndexer = new LexiconIndexer(new File(Environment.getInstance().getProperty("lexicaDirectory")), database);
		//lexiconIndexer.fsToDb();
	}

	public void fsToDb() {
		/*
		System.out.println("Reading lexica from files:");
		String name;
		int pos;
		Language source;
		Language target;
		try {
			for (File lexiconFile : rootFolder.listFiles()) {
				name = lexiconFile.getName();
				pos = name.indexOf(".");
				if (pos != -1) {
					name = name.substring(0, pos);
				}
				pos = name.indexOf("-");
				source = Language.valueOf(name.substring(0, pos));
				target = Language.valueOf(name.substring(pos + 1));
				if (source == null || target == null) {
					continue;
				}
				readLexicon(source, target, lexiconFile);
			}
			lexiconService.commit();
		} catch(NullPointerException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
			System.err.println("Coult not store lexica.");
		}
		*/
	}

	public void readLexicon(Language source, Language target) {
		File lexiconFile = new File(rootFolder.getPath() + "/" + source + "-" + target + ".txt");
		if (lexiconFile.exists()) {
			System.out.println("Importing " + source + " to " + target + " dictionary.");
			readLexicon(source, target, lexiconFile);
		}
	}

	private void readLexicon(Language source, Language target, File lexiconFile) {
		try {
			FileReader fr = new FileReader(lexiconFile);
			BufferedReader br = new BufferedReader(fr);
			String line, word, pos;
			int p, b1, b2;
			while ((line = br.readLine()) != null) {
				p = line.indexOf(":");
				if (p == -1) {
					continue;
				}
				word = line.substring(0, p).trim();
				pos = null;
				b1 = word.lastIndexOf("[");
				b2 = word.lastIndexOf("]");
				if (b1 > 0 && b2 > b1) {
					pos = word.substring(b1 + 1, b2).trim();
					word = word.replaceFirst(Pattern.quote("[" + pos + "]"), "").trim();
				}
				word = Useful.join(getStopper(source).process(Arrays.asList(word)), " ");
				line = line.substring(p + 1).trim();
				if ("".equals(line)) {
					continue;
				}
				//System.out.println(source + " " + target);
				//System.out.println(word + " " + line);
				//System.out.println(Arrays.toString(line.replaceAll("::", ";;").split(";;")));
				for (String split : line.replaceAll("::", ";;").split(";;")) {
					for (String equal : getStopper(target).process(new ArrayList<String>(getAllCombinations(split)))) {
						//System.out.println(equal);
					 lexiconService.addTranslation(source, target, word, pos,equal);
					}
				}
			}
			br.close();
			fr.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	private Set<String> getAllCombinations(String input) {
		int opens = 0;
		int closes = 0;
		int level = 0;
		loop:
		for (int i = 0; i < input.length(); i++) {
			switch (input.charAt(i)) {
				case '(':
					if (level++ == 0) {
						opens = i;
					}
					break;
				case ')':
					if (--level == 0) {
						closes = i;
						break loop;
					}
					break;
			}
		}
		Set<String> combos = new HashSet<String>();
		if (opens != closes) {
			if (closes - opens > 0) {
				//System.out.println("\t" + opens + " " + closes);
				combos.addAll(getAllCombinations(input.replace(
						input.substring(opens, closes + 1),
						""
				)));
			}
			if (closes - opens > 1) {
				combos.addAll(getAllCombinations(input.replace(
						input.substring(opens, closes + 1),
						input.substring(opens + 1, closes)
				)));
			}
		}
		else {
			Pattern p = Pattern.compile("([\\p{L}\\p{N}]+)\\/([\\p{L}\\p{N}]+)");
			Matcher m = p.matcher(input);
			if (m.find()) {
				//System.out.println("\ttest " + m.group(0) + " " + m.group(1) + " " + m.group(2));
				combos.add(input.replaceFirst(Pattern.quote(m.group(0)), m.group(1)));
				combos.add(input.replaceFirst(Pattern.quote(m.group(0)), m.group(2)));
			}
			else {
				combos.add(input);
			}
		}
		Set<String> trimmed = new HashSet<String>();
		for (String cmb : combos) {
			trimmed.add(cmb.replaceAll("\\s+", " ").trim());
		}
		return trimmed;
	}

	private Stopword getStopper(Language language) {
		if (!stoppers.containsKey(language)) {
			stoppers.put(language, new Stopword(language));
		}
		return stoppers.get(language);
	}
}
