package impl.indexing.extracting;

import impl.model.Useful;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

/**
 * This file is intended to extract xml based lexica provided by NTNU into plain text files.
 */
public class NtnuLexiconExtractor {
	private static Map<String, String> conversations;

	public static void main(String[] args) {
		String basePath = "../clir-lexica/";
		Map<String, String> subjects = new HashMap<String, String>();
		subjects.put("ntnu/norsk-engelsk.xml", "extracted/NO-EN.txt");
		subjects.put("ntnu/norsk-spansk.xml",  "extracted/NO-ES.txt");
		subjects.put("ntnu/norsk-tysk.xml",    "extracted/NO-DE.txt");
		subjects.put("ntnu/engelsk-norsk.xml", "extracted/EN-NO.txt");
		subjects.put("ntnu/spansk-norsk.xml",  "extracted/ES-NO.txt");
		subjects.put("ntnu/tysk-norsk.xml",    "extracted/DE-NO.txt");
		for (Map.Entry<String, String> entry : subjects.entrySet()) {
			File source = new File(basePath + entry.getKey());
			File target = new File(basePath + entry.getValue());
			extract(source, target);
		}
	}

	private static void extract(File source, File target) {
		if (!source.canRead()) {
			System.err.println("Can't read: " + source);
			return;
		}
		String debug = target.getName();
		debug = debug.substring(0, debug.indexOf("."));
		SAXBuilder sb = new SAXBuilder();
		try {
			PrintWriter pw = new PrintWriter(target);
			org.jdom2.Document document = sb.build(source);
			Element parseItExport = document.getRootElement();
			for (Element peArticle : parseItExport.getChildren()) {
				Element artikkel = peArticle.getChild("pe_content").getChild("artikkel");
				if (artikkel == null) {
					System.err.println(debug + ": " + artikkel);
					continue;
				}
				Element hode = artikkel.getChild("hode");
				if (hode == null) {
					System.err.println(debug + ": " + hode);
					continue;
				}
				Element oppslagsord = hode.getChild("oppslagsord");
				Element ordklasse = hode.getChild("ordklasse");
				String enOrdklasse = null;
				if (ordklasse != null) {
					enOrdklasse = ordklasse.getText();
					if (enOrdklasse.indexOf(",") != -1) {
						enOrdklasse = enOrdklasse.substring(0, enOrdklasse.indexOf(","));
					}
				}
				int i = 0;
				String line = oppslagsord.getText() + (enOrdklasse == null ? "" : "[" + convertPos(enOrdklasse) + "]") + ": ";
				Element betydningsseksjon = artikkel.getChild("betydningsseksjon");
				if (betydningsseksjon == null) {
					System.err.println(debug + ": " + oppslagsord.getText() + ": betydningsseksjon");
					continue;
				}
				for (Element betydning : betydningsseksjon.getChildren()) {
					if (i++ > 0) {
						line += " ;; ";
					}
					int j = 0;
					Element oversettelse = betydning.getChild("oversettelse");
					if (oversettelse == null) {
						System.err.println(debug + ": " + oppslagsord.getText() + ": oversettelse");
						continue;
					}
					XMLOutputter xo = new XMLOutputter();
					xo.setFormat(Format.getCompactFormat());
					Element ekvivalentgruppe = oversettelse.getChild("ekvivalentgruppe");
					if (ekvivalentgruppe == null) {
						Element ekvivalent = oversettelse.getChild("ekvivalent");
						if (ekvivalent != null) {
							StringWriter sw = new StringWriter();
							xo.output(ekvivalent.getContent(), sw);
							line += Useful.stripElements(sw.getBuffer().toString());
							continue;
						}
						System.err.println(debug + ": " + oppslagsord.getText() + ": ekvivalentgruppe || ekvivalent");
						continue;
					}
					for (Element ekvivalent : ekvivalentgruppe.getChildren()) {
						if (!"ekvivalent".equals(ekvivalent.getName())) {
							continue;
						}
						if (j++ > 0) {
							line += " :: ";
						}
						StringWriter sw = new StringWriter();
						xo.output(ekvivalent.getContent(), sw);
						line += Useful.stripElements(sw.getBuffer().toString());
					}
				}
				pw.println(line);
			}
			pw.close();
		} catch (JDOMException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static String convertPos(String original) {
		if (conversations == null) {
			conversations = new HashMap<String, String>();
			conversations.put("sb", "noun");
			conversations.put("m", "noun");
			conversations.put("f", "noun");
			conversations.put("ubest art", "article");
			conversations.put("adv", "adverb");
			conversations.put("vb", "verb");
			conversations.put("adj", "adjective");
			conversations.put("prep", "preposition");
			conversations.put("konj", "conjunction");
			conversations.put("interj", "interjection");
			conversations.put("pron", "pronoun");
		}
		return conversations.get(original);
	}
}