package impl.language;

import impl.language.tagging.OsloBergenTagger;
import impl.language.tagging.Tagger;
import impl.language.tagging.TreeTagger;
import impl.model.Term;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Preprocessor {
	private Language language;
	private Stopword stopword;
	private static Map<Language, Tagger> tagger = null;

	public Preprocessor(Language language) {
		this.language = language;
		stopword = new Stopword(language);
	}

	public List<Term> process(List<String> tokenList) {
		tokenList = stopword.process(tokenList);
		Tagger tagger = getTagger();
		if (tagger == null) {
			return new ArrayList<Term>();
		}
		List<Term> terms = tagger.process(tokenList);
		for (Term term : terms) {
			term.lemma = term.lemma.toLowerCase();
		}
		return terms;
	}

	private Tagger getTagger() {
		if (tagger == null) {
			tagger = new HashMap<Language, Tagger>();
		}
		if (!tagger.containsKey(language)) {
			do {
				if (language.equals(Language.NO) && OsloBergenTagger.isSupported()) {
					tagger.put(language, new OsloBergenTagger());
					break;
				}
				TreeTagger tt = new TreeTagger(language);
				if (tt.isSupported()) {
					tagger.put(language, tt);
					break;
				}
				tagger.put(language, null);
			} while(false);
		}
		return tagger.get(language);
	}
}
