package impl.language;

import org.tartarus.snowball.SnowballStemmer;

import java.util.HashMap;
import java.util.Map;

import org.tartarus.snowball.ext.englishStemmer;
import org.tartarus.snowball.ext.germanStemmer;
import org.tartarus.snowball.ext.norwegianStemmer;
import org.tartarus.snowball.ext.spanishStemmer;

public class Stemmer {
	private Language language;
	private static Map<Language, SnowballStemmer> snowball = null;

	public Stemmer(Language language) {
		this.language = language;
	}

	public String process(String token) {
		SnowballStemmer stemmer = getSnowball();
		stemmer.setCurrent(token);
		stemmer.stem();
		return stemmer.getCurrent();
	}

	public String[] process(String[] tokens) {
		if (language == null || tokens.length == 0) {
			return tokens;
		}
		String[] stemmed = new String[tokens.length];
		int i = 0;
		for (String token : tokens) {
			stemmed[i++] = process(token);
		}
		return stemmed;
	}

	private SnowballStemmer getSnowball() {
		if (snowball == null) {
			snowball = new HashMap<Language, SnowballStemmer>();
		}
		if (!snowball.containsKey(language)) {
			switch (language) {
				case EN:
					snowball.put(language, new englishStemmer());
					break;
				case DE:
					snowball.put(language, new germanStemmer());
					break;
				case NO:
					snowball.put(language, new norwegianStemmer());
					break;
				case ES:
					snowball.put(language, new spanishStemmer());
					break;
				default:
					return null;
			}
		}
		return snowball.get(language);
	}
}
