package impl.language;

import impl.model.Term;

import java.io.*;
import java.util.*;

public class Stopword {
	private Language language;
	private Set<String> words;

	public Stopword(Language language) {
		this.language = language;
		words = null;
	}

	public List<String> process(List<String> input) {
		List<String> output = new ArrayList<String>();
		for (String term : input) {
			term = term.replaceAll("[^\\p{L}\\p{N}]+", " ");
			//term = Normalizer.normalize(term, Normalizer.Form.NFKD);
			//term = term.replaceAll("[\\p{M}]+", "");
			//term = term.replaceAll("[^a-zA-Z0-9\\s]", "");
			for (String part : term.split("\\s+")) {
				part = part.toLowerCase().trim();
				if (part.length() == 0 || getWords().contains(part)) {
					continue;
				}
				output.add(part);
			}
		}
		return output;
	}

	public List<Term> processTerms(List<Term> input) {
		List<Term> output = new ArrayList<Term>();
		for (Term term : input) {
			term.lemma = term.lemma.replaceAll("[^\\p{L}\\p{N}]+", " ");
			//term = Normalizer.normalize(term, Normalizer.Form.NFKD);
			//term = term.replaceAll("[\\p{M}]+", "");
			//term = term.replaceAll("[^a-zA-Z0-9\\s]", "");
			for (String part : term.lemma.split("\\s+")) {
				part = part.toLowerCase().trim();
				if (part.length() == 0 || getWords().contains(part)) {
					continue;
				}
				output.add(new Term(part, term.pos));
			}
		}
		return output;
	}

	public Set<String> getWords() {
		if (words == null) {
			try {
				words = new HashSet<String>();
				FileReader fr = new FileReader(new File("stopword/" + language.toString() + ".stopword"));
				BufferedReader br = new BufferedReader(fr);
				String line;
				while ((line = br.readLine()) != null) {
					words.add(line.replaceFirst("\\|(.*)", "").trim().toLowerCase());
				}
				br.close();
				fr.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return words;
	}

	public static void main(String[] args) {
		Stopword stop = new Stopword(Language.NO);
		String[] terms = new String[]{"jeg", "er", "ikke", "noe", "litt", "bare", "ingen", "ting"};
		List<String> res = stop.process(Arrays.asList(terms));
		System.out.println(Arrays.toString(res.toArray(new String[0])));
	}
}
