package impl.language.splitting;

import impl.language.tagging.Tagger;
import impl.model.Term;

import java.util.*;

public class AbstractSplitter {
	private Tagger tagger;
	private Map<String, String> lemmas;

	public AbstractSplitter(Tagger tagger) {
		this.tagger = tagger;
		lemmas = new HashMap<String, String>();
	}

	public AbstractSplitter() {
	}

	protected void retrieveLemmas(Set<String[]> candidates) {
		Set<String> parts = new HashSet<String>();
		for (String[] cand : candidates) {
			for (String word : cand) {
				parts.add(word.toLowerCase());
			}
		}
		parts.removeAll(lemmas.keySet());
		List<String> before = new ArrayList<String>(parts);
		int i = 0;
		for (Term term : tagger.process(before)) {
			//System.out.println(before.get(i) + " " + term.lemma);
			lemmas.put(before.get(i++), term.lemma);
		}
	}

	protected String lemma(String word) {
		if (!lemmas.containsKey(word.toLowerCase())) {
			System.err.println("SLOW! " + word);
		}
		return lemmas.get(word.toLowerCase());
	}
}
