package impl.language.tagging;

import impl.model.Environment;
import impl.model.Term;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

public class OsloBergenTagger implements Tagger {
	private static String directory = null;
	private static Map<String, String> conversations = null;

	private static String getDirectory() {
		if (directory == null) {
			directory = Environment.getInstance().getProperty("oslobergentagger.directory");
		}
		return directory;
	}

	public List<Term> process(List<String> terms) {
		if (terms.size() > 10000) {
			int length = terms.size();
			List<Term> chunked = new ArrayList<Term>();
			chunked.addAll(process(terms.subList(0, length / 2)));
			chunked.addAll(process(terms.subList(length / 2, length)));
			return chunked;
		}
		try {
			//System.out.println("calling obtagger");
			StringBuilder sb = new StringBuilder();
			for (String item : terms) {
				sb.append(item + " ");
			}
			Process process = new ProcessBuilder(
					"script/obtagger.sh",
					sb.toString(),
					getDirectory()
			).start();
			InputStream is = process.getInputStream();
			InputStreamReader isr = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(isr);
			List<Term> termList = new ArrayList<Term>();
			String line, pos, lemma;
			String token = null;
			boolean entity;
			while ((line = br.readLine()) != null) {
				//System.out.println(line);
				if (token != null && line.startsWith("\t")) {
					lemma = line.substring(2, line.lastIndexOf("\""));
					pos = line.substring(line.lastIndexOf("\"") + 2);
					entity = false;
					if (pos.indexOf(" ") > 0) {
						if (pos.startsWith("subst prop")) {
							entity = true;
						}
						pos = pos.substring(0, pos.indexOf(" "));
					}
					Term term = new Term(lemma, convertPos(pos));
					term.isName = entity;
					termList.add(term);
					token = null;
				}
				else if (line.startsWith("<word>")) {
					token = line.substring(6, line.length() - 7);
				}
			}
			br.close();
			isr.close();
			is.close();
			process.destroy();
			return termList;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	public static String convertPos(String original) {
		if (conversations == null) {
			conversations = new HashMap<String, String>();
			conversations.put("adj", "adjective");
			conversations.put("adv", "adverb");
			conversations.put("det", "determiner");
			conversations.put("inf-merke", "unknown");
			conversations.put("interj", "interjection");
			conversations.put("konj", "conjunction");
			conversations.put("prep", "preposition");
			conversations.put("pron", "pronoun");
			conversations.put("sbu", "unknown");
			conversations.put("subst", "noun");
			conversations.put("ukjent", "unknown");
			conversations.put("verb", "verb");
			conversations.put("clb", "punctuation");
		}
		return conversations.get(original);
	}

	public static void main(String[] args) {
		if (!OsloBergenTagger.isSupported()) {
			return;
		}
		OsloBergenTagger obt = new OsloBergenTagger();
		List<Term> tagged = obt.process(Arrays.asList(new String[]{"Vibeke", "liker", "å", "spise", "kake", "hver", "dag", ".",
				"Og", "her", "er", "en", "\"annen\"", "setning", "til", "Vibeke", ".",
				"noun phrase", "error", "verb",
				"Ho", "Vibeke", "er", "adm. dir", "i", "Norsk Supperåd", "."}));
		for (Term item : tagged) {
			System.out.println(item.lemma + "\t" + item.pos + "\t" + item.isName);
		}
	}

	public static boolean isSupported() {
		String os = System.getProperty("os.name");
		if (os != null && os.toLowerCase().indexOf("windows") != -1) {
			System.err.println("Windows is not supported by the Oslo-Bergen tagger.");
			return false;
		}
		boolean missing = false;
		Process process;
		InputStream is;
		InputStreamReader isr;
		BufferedReader br;
		for (String cmd : Arrays.asList(getDirectory() + "/bin/mtag", "vislcg3", "perl")) {
			try {
				process = new ProcessBuilder(
						"which",
						cmd
				).start();
				is = process.getInputStream();
				isr = new InputStreamReader(is);
				br = new BufferedReader(isr);
				if (br.readLine() == null) {
					System.err.println(cmd + " is missing.");
					missing = true;
				}
				br.close();
				isr.close();
				is.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
		}
		if (missing) {
			return false;
		}
		return true;
	}
}
