package impl.language.tagging;

import impl.language.Language;
import impl.model.Environment;
import impl.model.Term;
import org.annolab.tt4j.TokenHandler;
import org.annolab.tt4j.TreeTaggerException;
import org.annolab.tt4j.TreeTaggerWrapper;

import java.io.*;
import java.util.*;

public class TreeTagger implements Tagger {
	private static boolean configOnce = false;

	private static void configOnce() {
		if (configOnce) {
			return;
		}
		configOnce = true;
		System.setProperty("treetagger.home", Environment.getInstance().getProperty("treetagger.home"));
	}

	private Language language;
	private String paramFile = null;
	private TreeTaggerWrapper wrapper;
	private Map<String, String> conversations = null;
	private List<Term> termList;

	public TreeTagger(Language language) {
		this.language = language;
	}

	public List<Term> process(List<String> terms) {
		try {
			configOnce();
			initWrapper();
			wrapper.setHandler(new TokenHandler<String>() {
				public void token(String token, String pos, String lemma) {
					termList.add(new Term(lemma, convertPos(pos)));
				}
			});
			termList = new ArrayList<Term>();
			wrapper.process(terms);
			return termList;
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TreeTaggerException e) {
			e.printStackTrace();
		}
		return null;
	}

	private void initWrapper() {
		if (wrapper != null) {
			return;
		}
		try {
			wrapper = new TreeTaggerWrapper<String>();
			wrapper.setModel(getParamFile());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private String getParamFile() {
		if (paramFile == null) {
			paramFile = Environment.getInstance().getProperty("treetagger." + language.toString());
		}
		return paramFile;
	}

	public boolean isSupported() {
		return getParamFile() != null;
	}

	private String convertPos(String original) {
		if (conversations == null) {
			try {
				conversations = new HashMap<String, String>();
				FileReader fr = new FileReader(new File("tagset/" + language.toString() + ".tagset"));
				BufferedReader br = new BufferedReader(fr);
				String line;
				String[] split;
				while ((line = br.readLine()) != null) {
					split = line.trim().split("\\s+");
					conversations.put(split[0], split[1]);
				}
				br.close();
				fr.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return conversations.get(original);
	}

	public static void main(String[] args) {
		List<String> testTerms = Arrays.asList(new String[]{"Vibeke", "likes", "to", "eat", "cake", "every", "day", ".",
				"And", "here", "is", "another", "sentence", "for", "Vibeke", ".",
				"noun phrase", "error", "verb"});
		TreeTagger tt = new TreeTagger(Language.EN);
		List<Term> tagged = tt.process(testTerms);
		for (Term item : tagged) {
			System.out.println(item.lemma + "\t" + item.pos);
		}
	}
}
