package impl.model;

import impl.language.Language;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

abstract class AbstractDocument {
	protected Language language;
	protected Map<Term, Double> vector;
	protected Map<Term, Double> normalisedVector;

	public void addTerm(Term term) {
		if (term == null) {
			return;
		}
		addTerm(term, 1);
	}

	public void addTerm(Term term, double count) {
		if (vector.containsKey(term)) {
			vector.put(term, vector.get(term) + count);
		}
		else {
			vector.put(term, count);
		}
	}

	public void addTerms(List<Term> termList) {
		for (Term term : termList) {
			addTerm(term);
		}
	}

	public Language getLanguage() {
		return language;
	}

	public Map<Term, Double> getVector() {
		return vector;
	}

	public Map<Term, Double> getNormalisedVector() {
		if (normalisedVector == null) {
			double sum = .0;
			normalisedVector = new HashMap<Term, Double>();
			for (Map.Entry<Term, Double> entry : vector.entrySet()) {
				sum += entry.getValue();
			}
			for (Map.Entry<Term, Double> entry : vector.entrySet()) {
				normalisedVector.put(entry.getKey(), entry.getValue() / sum);
			}
		}
		return normalisedVector;
	}
}