package impl.model;

import impl.language.Language;

import java.util.*;

public class Corpus extends AbstractDocument implements Document {
	private int id;
	private String collection;
	private String title;
	private String source;

	public Corpus(String collection, Language language, String title, String source) {
		this.id = 0;
		this.collection = collection;
		this.language = language;
		this.title = title;
		this.source = source;
		this.vector = new HashMap<Term, Double>();
	}

	public void orderVector() {
		List<Map.Entry<Term, Double>> ordered = new LinkedList<Map.Entry<Term, Double>>(vector.entrySet());
		Collections.sort(ordered, new Comparator<Map.Entry<Term, Double>>() {
			public int compare(Map.Entry<Term, Double> o1, Map.Entry<Term, Double> o2) {
				return o2.getValue().compareTo(o1.getValue());
			}
		});
		vector = new LinkedHashMap<Term, Double>();
		for (Map.Entry<Term, Double> entry : ordered) {
			vector.put(entry.getKey(), entry.getValue());
		}
	}

	public String toString() {
		String print = title + " " + language + "/" + collection + " (" + vector.size() + " words";
		if (id != 0) {
			print += ", id " + id;
		}
		print += ", source " + source + ")";
		/*
		for (Map.Entry<String, Integer> entry : vector.entrySet()) {
			print += entry.getKey() + "\t\t\t" + entry.getValue() + "\n";
		}
		*/
		return print;
	}

	public String getCollection() {
		return collection;
	}

	public String getTitle() {
		return title;
	}

	public String getSource() {
		return source;
	}

	public void setId(int id) {
		this.id = id;
	}

	public void setLanguage(Language language) {
		this.language = language;
	}

	public String toJson() {
		StringBuilder sb = new StringBuilder();
		sb.append("{\"collection\":\"" + collection + "\",");
		sb.append("\"language\":\"" + language + "\",");
		sb.append("\"title\":\"" + title + "\",");
		sb.append("\"source\":\"" + source + "\",");
		sb.append("\"vector\":{");
		/*
		int i = 0;
		for (Map.Entry<String, Integer> entry : vector.entrySet()) {
			if (i++ > 0) {
				sb.append(",");
			}
			sb.append("\"" + entry.getKey() + "\":" + entry.getValue());
		}
		*/
		sb.append("}}");
		return sb.toString();
	}
}
