package impl.model;

import impl.language.Language;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

public class CorpusServiceOld {
	private Database database;

	public CorpusServiceOld(Database database) {
		this.database = database;
	}

	public int addCorpus(Corpus corpus) {
		int corpusId = 0;
		int termId = 0;
		Term term = null;
		int counter = 0;
		try {
			final boolean oldAutoCommit = database.getConnection().getAutoCommit();
			try {
				database.getConnection().setAutoCommit(false);
				String sql = "INSERT INTO corpus(collection,language,title,source) VALUES(?,?,?,?);";
				PreparedStatement ps = database.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
				ps.setString(1, corpus.getCollection());
				ps.setString(2, corpus.getLanguage().toString());
				ps.setString(3, corpus.getTitle());
				ps.setString(4, corpus.getSource());
				ps.execute();
				ResultSet rs = ps.getGeneratedKeys();
				if (!rs.next()) {
					System.err.println("Corpus was not added: " + corpus);
					return 0;
				}
				corpusId = rs.getInt(1);
				for (Map.Entry<Term, Double> entry : corpus.getVector().entrySet()) {
					term = entry.getKey();
					termId = addTerm(corpus.getLanguage(), term);
					sql = "INSERT INTO corpus_term(corpus_id,term_id,count) VALUES(?,?,?);";
					ps = database.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
					ps.setInt(1, corpusId);
					ps.setInt(2, termId);
					ps.setDouble(3, entry.getValue());
					ps.execute();
					if (++counter % 1000 == 0) {
						System.out.print(counter + "\r");
					}
				}
				return corpusId;
			} catch (SQLException e) {
				e.printStackTrace();
				System.err.println("corpusId " + corpusId + ", termId " + termId + ", term \"" + term.lemma + "\" (" + term.pos + ")");
				database.getConnection().rollback();
			} finally {
				database.getConnection().commit();
				database.getConnection().setAutoCommit(oldAutoCommit);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return 0;
	}

	private int addTerm(Language language, Term term) {
		try {
			String sql = "SELECT id FROM term WHERE language=? AND lemma=? AND pos=?;";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setString(1, language.toString());
			ps.setString(2, term.lemma);
			ps.setString(3, term.pos);
			ResultSet rs = ps.executeQuery();
			if (rs.next()) {
				return rs.getInt(1);
			}
			else {
				sql = "INSERT INTO term(language,lemma,pos) VALUES(?,?,?);";
				ps = database.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
				ps.setString(1, language.toString());
				ps.setString(2, term.lemma);
				ps.setString(3, term.pos);
				ps.execute();
				rs = ps.getGeneratedKeys();
				if (rs.next()) {
					return rs.getInt(1);
				}
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return 0;
	}

	public Corpus getCorpus(int corpusId) {
		try {
			String sql = "SELECT * FROM corpus WHERE id=?;";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setInt(1, corpusId);
			ResultSet rs = ps.executeQuery();
			if (rs.next()) {
				Corpus instance = new Corpus(
						rs.getString("collection"),
						Language.valueOf(rs.getString("language")),
						rs.getString("title"),
						rs.getString("source")
				);
				instance.setId(corpusId);
				for (Map.Entry<Term, Integer> entry : getTerms(corpusId).entrySet()) {
					instance.addTerm(entry.getKey(), entry.getValue());
				}
				return instance;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	private Map<Term, Integer> getTerms(int corpusId) {
		try {
			String sql = "SELECT t.lemma AS lemma,pos,SUM(ct.count) AS count FROM term AS t,corpus_term AS ct WHERE t.id=ct.term_id AND ct.corpus_id=? GROUP BY t.id;";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setInt(1, corpusId);
			ResultSet rs = ps.executeQuery();
			Map<Term, Integer> terms = new HashMap<Term, Integer>();
			while (rs.next()) {
				terms.put(new Term(rs.getString("lemma"), rs.getString("pos")), rs.getInt("count"));
			}
			return terms;
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	public boolean exists(String collection, Language language, String title) {
		try {
			String sql = "SELECT id FROM corpus WHERE collection=? AND language=? AND title=?;";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setString(1, collection);
			ps.setString(2, language.toString());
			ps.setString(3, title);
			ResultSet rs = ps.executeQuery();
			if (rs.next()) {
				return true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return false;
	}
}
