package impl.model;

import impl.language.Language;

import java.util.*;

public class Lexicon {
	private Language source;
	private Language target;
	private Map<String, Map<Term, List<String>>> translations;

	public Lexicon(Language source, Language target) {
		this.source = source;
		this.target = target;
		translations = new HashMap<String, Map<Term, List<String>>>();
	}

	public void addTranslation(Term from, String to) {
		from.lemma = from.lemma.toLowerCase();
		to = to.toLowerCase();
		if (!translations.containsKey(from.lemma)) {
			translations.put(from.lemma, new HashMap<Term, List<String>>());
		}
		if (!translations.get(from.lemma).containsKey(from)) {
			translations.get(from.lemma).put(from, new ArrayList<String>());
		}
		translations.get(from.lemma).get(from).add(to);
	}

	public boolean hasWord(String word) {
		return translations.containsKey(word.toLowerCase()) && translations.get(word.toLowerCase()).entrySet().size() > 0;
	}

	public List<Term> getTranslations(Term from) {
		from.lemma = from.lemma.toLowerCase();
		//if (from.isName) {
		//	return Collections.singletonList(from);
		//}
		if (!translations.containsKey(from.lemma)) {
			return Collections.emptyList();
		}
		List<Term> termList = new ArrayList<Term>();
		//for (String lemma : translations.get(from.lemma).get(from)) {
		//	termList.add(new Term(lemma, from.pos));
		//}
		for (Map.Entry<Term, List<String>> entry : translations.get(from.lemma).entrySet()) {
			//if (from.equals(entry.getKey())) {
			//	continue;
			//}
			for (String lemma : entry.getValue()) {
				termList.add(new Term(lemma, entry.getKey().pos));
			}
		}
		return termList;
	}

	public void extend(Lexicon other) {
		for (Map<Term, List<String>> word : other.translations.values()) {
			for (Map.Entry<Term, List<String>> entry : word.entrySet()) {
				for (String equal : entry.getValue()) {
					addTranslation(entry.getKey(), equal);
				}
			}
		}
	}
}
