package impl.model;

import impl.indexing.LexiconIndexer;
import impl.language.Language;

import java.io.File;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

public class LexiconService {
	private Database database;
	private boolean oldAutoCommit;
	private boolean isTransacting;
	private int inserted;

	private Map<LanguagePair, Lexicon> lexica;

	public LexiconService(Database database) {
		this.database = database;
		isTransacting = false;
		inserted = 0;
		lexica = new HashMap<LanguagePair, Lexicon>();
	}

	private class LanguagePair {
		private Language source;
		private Language target;

		public LanguagePair(Language source, Language target) {
			this.source = source;
			this.target = target;
		}

		@Override
		public boolean equals(Object obj) {
			if (!(obj instanceof LanguagePair)) {
				return false;
			}
			if (obj == this) {
				return true;
			}
			LanguagePair other = ((LanguagePair) obj);
			return (source == null ? source == other.source : source.equals(other.source))
					&& (target == null ? target == other.target : target.equals(other.target));
		}

		@Override
		public int hashCode() {
			return ((source == null ? 0 : source.hashCode()) % 53 + 3) * ((target == null ? 0 : target.hashCode()) % 82 + 43);
		}
	}

	public void addTranslation(Language source, Language target, String word, String pos, String equal) throws SQLException {
		//System.out.println(source + " " + target + " " + word + " " + pos + " " + equal);
		getLexicon(source, target).addTranslation(new Term(word, pos), equal);
		if (++inserted % 20000 == 0) {
			System.out.println(inserted);
		}
		/*
		if (!isTransacting) {
			begin();
		}
		else if (inserted % 20000 == 0) {
			commit();
			begin();
		}
		try {
			String sql = "INSERT INTO translation(source,target,word,pos,equal) VALUES(?,?,?,?,?);";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setString(1, source.name());
			ps.setString(2, target.name());
			ps.setString(3, word);
			ps.setString(4, pos);
			ps.setString(5, equal);
			if (ps.executeUpdate() > 0) {
				System.out.println(++inserted + ". " + printable(source, target, word, pos, equal));
				ps.close();
			}
		} catch (SQLException e) {
			System.err.println("Not added: " + printable(source, target, word, pos, equal));
		}
		*/
	}

	public void begin() throws SQLException {
		/*
		isTransacting = true;
		oldAutoCommit = database.getConnection().getAutoCommit();
		database.getConnection().setAutoCommit(false);
		*/
	}

	public void commit() throws SQLException {
		/*
		isTransacting = false;
		database.getConnection().commit();
		database.getConnection().setAutoCommit(oldAutoCommit);
		*/
	}

	private String printable(Language source, Language target, String word, String pos, String equal) {
		return "(" + source + "->" + target + ") " + word + " (" + pos + ") -> " + equal;
	}

	public Lexicon getLexicon(Language source) {
		return getLexicon(source, null);
	}

	public Lexicon getLexicon(Language source, Language target) {
		if (target == null) {
			Lexicon lexicon = new Lexicon(source, null);
			for (Language lang : Language.values()) {
				lexicon.extend(getLexicon(source, lang));
			}
			return lexicon;
		}
		LanguagePair relation = new LanguagePair(source, target);
		if (lexica.containsKey(relation)) {
			return lexica.get(relation);
		}
		lexica.put(relation, new Lexicon(source, target));
		LexiconIndexer lexiconIndexer = new LexiconIndexer(new File(Environment.getInstance().getProperty("lexicaDirectory")), this);
		lexiconIndexer.readLexicon(source, target);
		return lexica.get(relation);
		/*
		try {
			String sql = "SELECT word,pos,equal FROM translation WHERE source=?" + (target == null ? "" : " AND target=?") + ";";
			PreparedStatement ps = database.getConnection().prepareStatement(sql);
			ps.setString(1, source.name());
			if (target != null) {
				ps.setString(2, target.name());
			}
			ResultSet rs = ps.executeQuery();
			Lexicon instance = new Lexicon(source, target);
			while (rs.next()) {
				instance.addTranslation(new Term(rs.getString("word"), rs.getString("pos")), rs.getString("equal"));
			}
			lexica.put(relation, instance);
			return instance;
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
		*/
	}
}