package impl.model;

import impl.language.Language;
import impl.language.Preprocessor;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Query extends AbstractDocument implements Document {
	public Query(Language language) {
		this.language = language;
		this.vector = new HashMap<Term, Double>();
		normalisedVector = null;
	}

	public static Query byString(String query, Language language) {
		Query instance = new Query(language);
		Preprocessor pp = new Preprocessor(language);
		instance.addTerms(pp.process(Arrays.asList(query.split("\\s+"))));
		return instance;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Query[");
		sb.append(language);
		sb.append("](");
		int i = 0;
		for (Map.Entry<Term, Double> entry : vector.entrySet()) {
			if (i++ > 0) {
				sb.append(",");
			}
			sb.append(entry.getKey());
		}
		sb.append(")");
		return sb.toString();
	}

	public String toJson() {
		StringBuilder sb = new StringBuilder();
		sb.append("{\"language\":\"" + language + "\",");
		sb.append("\"vector\":[");
		int i = 0;
		for (Map.Entry<Term, Double> entry : vector.entrySet()) {
			if (i++ > 0) {
				sb.append(",");
			}
			sb.append(entry.getKey().toJson());
		}
		sb.append("]}");
		return sb.toString();
	}
}
