package impl.model;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

public class SearchReport {
	private Database database;
	private CorpusService corpusService;
	private VectorSpace vectorSpace;
	private boolean grammar;
	private List<Query> queries;
	private List<ResultSet> resultSets;
	private int cursor;

	public SearchReport(Database database, boolean grammar) {
		this.database = database;
		vectorSpace = new VectorSpace(database);
		corpusService = new CorpusService(database);
		this.grammar = grammar;
		queries = new ArrayList<Query>();
		resultSets = new ArrayList<ResultSet>();
		cursor = 0;
	}

	public SearchReport(Database database, boolean grammar, Query query, ResultSet resultSet) {
		this.database = database;
		vectorSpace = new VectorSpace(database);
		corpusService = new CorpusService(database);
		this.grammar = grammar;
		queries = new ArrayList<Query>();
		queries.add(query);
		resultSets = new ArrayList<ResultSet>();
		resultSets.add(resultSet);
		cursor = 0;
	}

	public List<Query> getQueries() {
		return queries;
	}

	public List<ResultSet> getResultSets() {
		return resultSets;
	}

	public ResultSet getNextResult() {
		while (resultSets.size() > cursor && resultSets.get(cursor) == null) {
			cursor++;
		}
		if (cursor >= resultSets.size()) {
			return null;
		}
		try {
			while (resultSets.size() > cursor && !resultSets.get(cursor).next()) {
				resultSets.get(cursor++).close();
			}
			if (cursor >= resultSets.size()) {
				return null;
			}
			return resultSets.get(cursor);
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	public SearchResult getNextCorpus() {
		ResultSet rs = getNextResult();
		if (rs == null) {
			return null;
		}
		try {
			//System.out.println(resultSets.get(0).getInt(1) + " " + resultSets.get(0).getDouble(2));
			Corpus corpus = corpusService.getCorpus(rs.getInt(1));
			//System.out.println(queries.get(cursor).getLanguage() + " " + corpus.getLanguage());
			return new SearchResult(corpus, vectorSpace.cosSim(queries.get(cursor), corpus, grammar));
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return null;
	}

	public List<SearchResult> getCorpora() {
		List<SearchResult> corpora = new ArrayList<SearchResult>();
		SearchResult res;
		while ((res = getNextCorpus()) != null) {
			corpora.add(res);
		}
		return corpora;
	}

	public String toJson() {
		StringBuilder sb = new StringBuilder();
		sb.append("{\"queries\":[");
		int i = 0;
		for (Query q : queries) {
			if (i++ > 0) {
				sb.append(",");
			}
			sb.append(q.toJson());
		}
		sb.append("],");
		sb.append("\"corpora\":[");
		i = 0;
		for (SearchResult sr : getCorpora()) {
			if (i++ > 0) {
				sb.append(",");
			}
			sb.append(sr.toJson());
		}
		sb.append("]}");
		return sb.toString();
	}

	public void expand(SearchReport another) {
		if (grammar != another.grammar || !database.equals(another.database)) {
			throw new IllegalArgumentException("Database or grammar incompatible.");
		}
		queries.addAll(another.queries);
		resultSets.addAll(another.resultSets);
		cursor = 0;
	}

	/*
	public void orderByCosineSimilarity() {
		Collections.sort(corpora, new Comparator<SearchResult>() {
			public int compare(SearchResult o1, SearchResult o2) {
				return o2.getCosineSimilarity() - o1.getCosineSimilarity() > 0 ? 1 : -1;
			}
		});
	}
	*/
}
