package impl;

import impl.language.Language;
import impl.language.Translator;
import impl.model.*;

import java.util.Set;

import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;
import org.junit.Test;

import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

@RunWith(JUnit4.class)
public class TranslatorTest {
	@Test
	public void isTranslating() {
		Database database = new Database(Environment.getInstance().getProperty("database"));
		LexiconService ls = new LexiconService(database);
		Lexicon noToEs = ls.getLexicon(Language.NO, Language.ES);
		assertTrue(noToEs.hasWord("stor"));
		assertTrue(noToEs.hasWord("mann"));
		assertTrue(noToEs.hasWord("gal"));
		assertTrue(noToEs.getTranslations(new Term("stor", null)).get(0).lemma.equals("grande"));
		Query original = new Query(Language.NO);
		original.addTerm(new Term("stor", null));
		original.addTerm(new Term("mann", null));
		original.addTerm(new Term("gal", null));
		Translator translator = new Translator(database);
		Set<Query> result = translator.translate(original, Language.ES, false, Translator.Pruning.BALANCE);
		for (Query query : result) {
			assertTrue(query.getVector().containsKey(new Term("grande", "adjective")));
			assertTrue(query.getVector().containsKey(new Term("hombre", "noun")));
			assertTrue(query.getVector().containsKey(new Term("erróneo", "adjective")));
		}
		result = translator.translate(original, Language.ES, true, Translator.Pruning.BALANCE);
		for (Query query : result) {
			assertTrue(query.getVector().containsKey(new Term("grande", "adjective")));
			assertTrue(query.getVector().containsKey(new Term("hombre", "noun")));
			assertTrue(query.getVector().containsKey(new Term("erróneo", "adjective")));
		}
	}
}
